/* Scrutari - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets;

import net.mapeadores.scrutari.db.ScrutariDB;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.SctContext;
import fr.exemole.sctserver.request.RequestInfo;
import fr.exemole.sctserver.request.json.JsonProducerFactory;
import fr.exemole.sctserver.request.simple.SimpleParameterErrorException;
import fr.exemole.sctserver.request.simple.SimpleRequest;
import fr.exemole.sctserver.request.simple.SimpleRequestFactory;
import fr.exemole.sctserver.request.uri.URIRequest;
import fr.exemole.sctserver.request.xml.FichesRequest;
import fr.exemole.sctserver.request.xml.MetadataRequest;
import fr.exemole.sctserver.request.xml.MotsclesRequest;
import fr.exemole.sctserver.request.xml.SearchRequest;
import fr.exemole.sctserver.request.xml.XmlRequestException;
import fr.exemole.sctserver.servlets.admin.AdminRequest;
import fr.exemole.sctserver.servlets.explore.ExploreRequest;
import fr.exemole.sctserver.servlets.feed.FeedRequest;
import fr.exemole.sctserver.servlets.global.GlobalRequest;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.servlets.handlers.JsonResponseHandler;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.text.UTF8Bridge;
import net.mapeadores.util.text.StringUtils;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.servlets.DefaultRequestMap;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariServlet extends HttpServlet {

    @Override
    public void doGet(HttpServletRequest requete, HttpServletResponse reponse) throws IOException, ServletException {
        process(requete, reponse);
    }

    @Override
    public void doPost(HttpServletRequest requete, HttpServletResponse reponse) throws IOException, ServletException {
        process(requete, reponse);
    }

    private void process(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        String pathInfo = request.getPathInfo();
        if (pathInfo == null) {
            pathInfo = "/";
        }
        int idx = pathInfo.indexOf('/', 2);
        if (idx != -1) {
            String name = pathInfo.substring(1, idx);
            String sctserverPathInfo = pathInfo.substring(idx + 1);
            processSctServerRequest(name, sctserverPathInfo, request, response);
        } else {
            processGlobalRequest(pathInfo, request, response);
        }
    }

    private void processGlobalRequest(String path, HttpServletRequest request, HttpServletResponse response) throws IOException {
        ScrutariContext scrutariContext = ScrutariContext.getScrutariContext(getServletContext());
        StringBuffer requestURLString = request.getRequestURL();
        if (path.length() == 1) {
            if (requestURLString.charAt(requestURLString.length() - 1) != '/') {
                response.sendRedirect(requestURLString.append('/').toString());
                return;
            }
        }
        ResponseHandler responseHandler = null;
        if (path.endsWith(".css")) {
            responseHandler = ResponseHandlerFactory.getCssHandler(path);
        } else if (path.endsWith(".png")) {
            responseHandler = ResponseHandlerFactory.getPngHandler(path);
        } else {
            GlobalRequest globalRequest = GlobalRequest.init(path);
            if (globalRequest == null) {
                if (scrutariContext.getSctContext(path) != null) {
                    response.sendRedirect(requestURLString.append('/').toString());
                    return;
                }
            } else {
                globalRequest.run(getServletContext(), scrutariContext);
                responseHandler = globalRequest.getResponsePrinter();
            }
        }
        handleResponse(request, response, responseHandler);
    }

    private void processSctServerRequest(String name, String sctserverPathInfo, HttpServletRequest request, HttpServletResponse response) throws IOException {
        ScrutariContext scrutariContext = ScrutariContext.getScrutariContext(getServletContext());
        SctContext sctContext = scrutariContext.getSctContext(name);
        if (sctContext == null) {
            printNotFound(request, response);
            return;
        }
        if (sctserverPathInfo.length() == 0) {
            response.sendError(HttpServletResponse.SC_FORBIDDEN);
            return;
        }
        int idx = sctserverPathInfo.indexOf('/');
        if (idx == -1) {
            String minuscule = sctserverPathInfo.toLowerCase();
            if ((minuscule.equals("simple")) || (minuscule.equals("rchfplain"))) {
                printSimple(request, response, sctContext);
            } else if ((minuscule.equals("uri")) || (minuscule.equals("rchfuri"))) {
                printURI(request, response, sctContext);
            } else if ((minuscule.equals("xml")) || (minuscule.equals("rchfxml"))) {
                printXML(request, response, sctContext);
            } else if (minuscule.equals("json")) {
                printJSon(request, response, sctContext);
            } else {
                printNotFound(request, response);
                return;
            }
        } else {
            String servletPath = sctserverPathInfo.substring(0, idx);
            sctserverPathInfo = sctserverPathInfo.substring(idx);
            ResponseHandler responseHandler = null;
            ScrutariRequestMap requestMap = new ScrutariRequestMap(request);
            if (servletPath.equals("admin")) {
                AdminRequest adminRequest = AdminRequest.init(sctContext, sctserverPathInfo, requestMap);
                if (adminRequest != null) {
                    responseHandler = adminRequest.getResponseHandler();
                }
            } else if (servletPath.equals("explore")) {
                ExploreRequest exploreRequest = ExploreRequest.init(sctContext, sctserverPathInfo, requestMap);
                if (exploreRequest != null) {
                    responseHandler = exploreRequest.getResponseHandler();
                }
            } else if (servletPath.equals("feed")) {
                FeedRequest feedRequest = FeedRequest.init(sctContext, sctserverPathInfo);
                if (feedRequest != null) {
                    responseHandler = feedRequest.getResponseHandler();
                }
            } else if (servletPath.equals("css")) {
                responseHandler = ResponseHandlerFactory.getCssHandler(sctserverPathInfo);
            } else if (servletPath.equals("dtd")) {
                responseHandler = ResponseHandlerFactory.getDtdHandler(sctserverPathInfo);
            } else if (servletPath.equals("js")) {
                responseHandler = ResponseHandlerFactory.getJsHandler(sctserverPathInfo);
            } else if (servletPath.equals("logs")) {
                if (sctserverPathInfo.equals("/")) {
                    responseHandler = new DirectoryIndexPage(sctContext.getSctServer(), "logs");
                } else {
                    responseHandler = ResponseHandlerFactory.getLogsHandler(sctserverPathInfo, sctContext.getSctServer().getLogAccess());
                }
            }
            handleResponse(request, response, responseHandler);
        }
    }

    private void handleResponse(HttpServletRequest request, HttpServletResponse response, ResponseHandler responseHandler) throws IOException {
        if (responseHandler == null) {
            printNotFound(request, response);
            return;
        }
        long lastModified = responseHandler.getLastModified();
        if (lastModified == ResponseHandler.LASTMODIFIED_NOCACHE) {
            response.setHeader("Cache-Control", "no-store");
        } else if (lastModified > 0) {
            response.setDateHeader("Last-Modified", lastModified);
            try {
                long since = request.getDateHeader("If-Modified-Since");
                if (since >= lastModified) {
                    response.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
                    return;
                }
            } catch (IllegalArgumentException iae) {
            }
        }
        responseHandler.handleResponse(response);
    }

    private void printSimple(HttpServletRequest requete, HttpServletResponse reponse, SctContext sctContext) throws IOException {
        reponse.setHeader("Cache-Control", "no-store");
        reponse.setContentType("text/plain;charset=UTF-8");
        DefaultRequestMap requestMap = new DefaultRequestMap(requete);
        try {
            SimpleRequest plainRequest = SimpleRequestFactory.getSimpleRequest(sctContext.getSctServer(), requestMap);
            plainRequest.printResponse(reponse.getWriter());
        } catch (SimpleParameterErrorException ppee) {
            PrintWriter pw = reponse.getWriter();
            pw.print("!-");
            pw.print(String.valueOf(ppee.getType()));
            String parameterName = ppee.getParameterName();
            if (parameterName != null) {
                pw.print('\n');
                pw.print("parameterName: ");
                pw.print(parameterName);
            }
            String parameterValue = ppee.getParameterValue();
            if (parameterValue != null) {
                pw.print('\n');
                pw.print("parameterValue: ");
                pw.print(parameterValue);
            }
            pw.close();

        }
    }

    private void printURI(HttpServletRequest requete, HttpServletResponse reponse, SctContext sctContext) throws IOException {
        reponse.setHeader("Cache-Control", "no-store");
        String param_uri = requete.getParameter("uri");
        if (param_uri != null) {
            URIRequest uriRequest = new URIRequest(param_uri, sctContext.getSctServer().getScrutariDB());
            uriRequest.printResponse(reponse);
            return;
        }
    }

    private void printJSon(HttpServletRequest requete, HttpServletResponse reponse, SctContext sctContext) throws IOException {
        DefaultRequestMap requestMap = new DefaultRequestMap(requete);
        JsonProducer jsonProducer = JsonProducerFactory.getJsonProducer(sctContext.getSctServer(), requestMap);
        JsonResponseHandler jsonResponseHandler = new JsonResponseHandler(jsonProducer, requestMap.getParameter("callback"));
        jsonResponseHandler.handleResponse(reponse);
    }

    private void printXML(HttpServletRequest requete, HttpServletResponse reponse, SctContext sctContext) throws IOException {
        reponse.setHeader("Cache-Control", "no-store");
        UTF8Bridge bridge = new UTF8Bridge(requete.getCharacterEncoding());
        RequestInfo requestInfo = newRequestInfo(requete);
        try {
            SctServer sctServer = sctServer = sctContext.getSctServer();
            String param_cmd = requete.getParameter("cmd");
            String param_xml = bridge.toUTF8(requete.getParameter("xml"));
            String param_searchid = requete.getParameter("searchid");
            if (param_cmd == null) {
                throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.MISSING_CMD_PARAMETER);
            }
            ScrutariDB generalDatabase = sctServer.getScrutariDB();
            if (param_cmd.equals("metadata")) {
                MetadataRequest metadataRequest = new MetadataRequest(generalDatabase);
                metadataRequest.printResponse(reponse);
            } else if (param_cmd.equals("search")) {
                SearchRequest searchRequest = new SearchRequest(param_xml, sctServer, requestInfo);
                searchRequest.printResponse(reponse);
            } else if (param_cmd.equals("fiches")) {
                FichesRequest fichesRequest = new FichesRequest(param_searchid, sctServer);
                fichesRequest.printResponse(reponse);
            } else if (param_cmd.equals("motscles")) {
                MotsclesRequest motsclesRequest = new MotsclesRequest(param_searchid, sctServer);
                motsclesRequest.printResponse(reponse);

            } else {
                throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.WRONG_CMD_PARAMETER, param_cmd);
            }
        } catch (XmlRequestException xre) {
            xre.printErrorResponse(reponse);
            Throwable cause = xre.getCause();
            if (cause != null) {
                cause.printStackTrace();
            }
        }
    }

    private void printNotFound(HttpServletRequest request, HttpServletResponse response) throws IOException {
        response.sendError(HttpServletResponse.SC_NOT_FOUND, request.getPathInfo());
    }

    /* à transférer */
    public final static String LOG_PARAMETER = "log";
    public final static String SITE_PARAMETER = "site";
    public final static String REFERRER_PARAMETER = "referrer";

    public static RequestInfo newRequestInfo(HttpServletRequest requete) {
        String remoteAddress = requete.getRemoteAddr();
        String param_log = requete.getParameter(LOG_PARAMETER);
        boolean withLog = true;
        if ((param_log != null) && (param_log.equals("0"))) {
            withLog = false;
        }
        String site = requete.getParameter(SITE_PARAMETER);
        if (site != null) {
            if (!StringUtils.isTechnicalName(site, true)) {
                site = null;
            }
        }
        String referrer = requete.getParameter(REFERRER_PARAMETER);
        return new RequestInfo(remoteAddress, withLog, site, referrer);
    }

}
