/* Scrutari - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.contentstreams;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.servlets.selection.SelectionUnit;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.opendocument.io.OdContentWriter;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.opendocument.io.ZipEntrySource;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.scrutari.db.CountStats;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.LexieAccess;
import net.mapeadores.scrutari.db.stats.MotcleCountStats;
import net.mapeadores.scrutari.db.util.LibelleUtils;
import net.mapeadores.scrutari.lexie.LexieOccurrencesList;
import net.mapeadores.scrutari.lexie.ScrutariLexieUnit;
import net.mapeadores.scrutari.lexie.ScrutariLexieUnitList;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.text.Libelle;
import net.scrutari.datauri.ThesaurusURI;


/**
 *
 * @author Vincent Calame
 */
public class LexiesOdsStream implements ResponseHandler {

    private SctServer sctServer;
    private int libLangInteger;
    private SelectionUnit selectionUnit;
    private int workingLangInteger;

    public LexiesOdsStream(SctServer sctServer, LocalisationInfo localisationInfo, int langInteger, SelectionUnit selectionUnit) {
        this.sctServer = sctServer;
        this.libLangInteger = langInteger;
        this.selectionUnit = selectionUnit;
        this.workingLangInteger = localisationInfo.getWorkingLangInteger();
    }

    public long getLastModified() {
        return ResponseHandler.LASTMODIFIED_NOCACHE;
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType(OdUtils.getMimeType(OdUtils.OD_SPREADSHEET));
        OutputStream outputStream = response.getOutputStream();
        LexieOdsWriter lexieOdsWriter = new LexieOdsWriter();
        lexieOdsWriter.export(outputStream);
        outputStream.close();

    }


    private class LexieOdsWriter {

        private OdZip odZip;

        LexieOdsWriter() {
            odZip = new OdZip();
            odZip.setOdtype(OdUtils.OD_SPREADSHEET);
            odZip.setContentZipEntrySource(new ContentZipEntrySource());
        }

        public void export(OutputStream outputStream) throws IOException {
            odZip.zip(outputStream);
        }


        private class ContentZipEntrySource implements ZipEntrySource {

            public void writeEntry(OutputStream outputStream) throws IOException {
                BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
                LexieXMLWriter xmlWriter = new LexieXMLWriter();
                xmlWriter.setAppendable(writer);
                xmlWriter.setIndentLength(0);
                xmlWriter.init();
                xmlWriter.start();
                xmlWriter.printTable();
                xmlWriter.end();
                writer.flush();
            }


            private class LexieXMLWriter extends OdContentWriter {

                ThesaurusData[] thesaurusDataArray;
                String[] baseLibelle;
                String[] thesaurusLibelle;
                int[] codethesaurusArray;
                String[] name;
                int thesaurusCount;
                CountStats countStats;
                DataAccess dataAccess;
                LexieAccess lexieAccess;
                Map<String, LexieBundle> bundleMap = new TreeMap<String, LexieBundle>();

                LexieXMLWriter() {
                }

                void init() {
                    dataAccess = sctServer.getScrutariDB().openDataAccess();
                    lexieAccess = sctServer.getScrutariDB().openLexieAccess();
                    countStats = sctServer.getScrutariDB().getCountStats();
                    int thesaurusMaxCount = dataAccess.getThesaurusCount();
                    thesaurusDataArray = new ThesaurusData[thesaurusMaxCount];
                    name = new String[thesaurusMaxCount];
                    thesaurusLibelle = new String[thesaurusMaxCount];
                    baseLibelle = new String[thesaurusMaxCount];
                    codethesaurusArray = new int[thesaurusMaxCount];
                    int p = 0;
                    for (int i = 0; i < thesaurusMaxCount; i++) {
                        ThesaurusData thData = dataAccess.getThesaurusDataByIndex(i);
                        ThesaurusURI thesaurusURI = thData.getThesaurusURI();
                        if ((selectionUnit == null) || (selectionUnit.contains(thesaurusURI))) {
                            thesaurusDataArray[p] = thData;
                            codethesaurusArray[p] = thData.getCodeThesaurus();
                            name[p] = thesaurusURI.getThesaurusName();
                            thesaurusLibelle[p] = LibelleUtils.getLibelleString(thData, workingLangInteger, "?");
                            BaseData baseData = dataAccess.getBaseDataByCode(thData.getCodeBase());
                            baseLibelle[p] = LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_SHORT, workingLangInteger, "?");
                            p++;
                        }
                    }
                    thesaurusCount = p;
                    initBundleMap();

                }

                private void initBundleMap() {
                    ScrutariLexieUnitList list = lexieAccess.getScrutariLexieUnitList(libLangInteger);
                    int count = list.getScrutariLexieUnitCount();
                    for (int i = 0; i < count; i++) {
                        ScrutariLexieUnit scrutariLexieUnit = list.getScrutariLexieUnit(i);
                        if (scrutariLexieUnit.hasMotcleAndLexieLink()) {
                            initLexieBundle(scrutariLexieUnit);
                        }
                    }
                }

                private void initLexieBundle(ScrutariLexieUnit lexieUnit) {
                    String canonicalLexie = lexieUnit.getCanonicalLexie();
                    LexieBundle lexieBundle = new LexieBundle(canonicalLexie);
                    boolean avec = false;
                    long lexieId = lexieUnit.getLexieId();
                    for (int i = 0; i < thesaurusCount; i++) {
                        int codethesaurus = codethesaurusArray[i];
                        LexieOccurrencesList lexieOccurrencesList = lexieAccess.getLexieOccurrencesList(lexieId, codethesaurus);
                        int codeCount = lexieOccurrencesList.getLexieOccurrencesCount();
                        for (int j = 0; j < codeCount; j++) {
                            int codemotcle = lexieOccurrencesList.getLexieOccurrences(j).getCode();
                            MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
                            MotcleCountStats motcleCountStats = countStats.getMotcleCountStats(codemotcle);
                            int count = motcleCountStats.getFicheCount();
                            Libelle libelle = motcleData.getLibelleByLang(libLangInteger);
                            lexieBundle.add(i, libelle.getLibelleString(), count);
                            avec = true;
                        }
                    }
                    if (avec) {
                        bundleMap.put(lexieUnit.getCollatedLexie(), lexieBundle);
                    }
                }

                private void start() throws IOException {
                    appendXMLDeclaration();
                    openDocumentContent();
                    openBody();
                    openSpreadsheet();
                }

                private void printTable() throws IOException {
                    openTable("lexies_" + LangInteger.toString(libLangInteger));
                    for (LexieBundle lexieBundle : bundleMap.values()) {
                        printLexieBundle(lexieBundle);
                    }
                    closeTable();
                }

                private void printLexieBundle(LexieBundle lexieBundle) throws IOException {
                    String canonicalLexie = lexieBundle.getCanonicalLexie();
                    boolean premier = true;
                    for (MotcleBundle motcleBundle : lexieBundle.getMotcleBundleList()) {
                        printMotcleBundle(canonicalLexie, premier, motcleBundle);
                        premier = false;
                    }
                }

                private void printMotcleBundle(String canonicalLexie, boolean premier, MotcleBundle motcleBundle) throws IOException {
                    int thesaurusIndex = motcleBundle.getThesaurusIndex();
                    openTableRow();
                    if (premier) {
                        addStringTableCell(canonicalLexie);
                    } else {
                        addEmptyTableCell();
                    }
                    addStringTableCell(canonicalLexie);
                    addStringTableCell(motcleBundle.getLibelleString());
                    addStringTableCell(thesaurusLibelle[thesaurusIndex]);
                    addStringTableCell(baseLibelle[thesaurusIndex]);
                    addIntegerTableCell(motcleBundle.getCount());
                    closeTableRow();
                }

                private void end() throws IOException {
                    closeSpreadsheet();
                    closeBody();
                    closeDocumentContent();
                    dataAccess.close();
                    lexieAccess.close();
                }

            }

        }

    }


    private static class LexieBundle {

        private String canonicalLexie;
        private List<MotcleBundle> motcleBundleList = new ArrayList<MotcleBundle>();

        LexieBundle(String canonicalLexie) {
            this.canonicalLexie = canonicalLexie;
        }

        public void add(int thesaurusIndex, String libelleString, int count) {
            MotcleBundle motcleBundle = new MotcleBundle();
            motcleBundle.setThesaurusIndex(thesaurusIndex);
            motcleBundle.setLibelleString(libelleString);
            motcleBundle.setCount(count);
            motcleBundleList.add(motcleBundle);
        }

        public List<MotcleBundle> getMotcleBundleList() {
            return motcleBundleList;
        }

        public String getCanonicalLexie() {
            return canonicalLexie;
        }

    }


    private static class MotcleBundle {

        private int count;
        private int thesaurusIndex;
        private String libelleString;

        MotcleBundle() {
        }

        public int getCount() {
            return count;
        }

        public void setCount(int count) {
            this.count = count;
        }

        public int getThesaurusIndex() {
            return thesaurusIndex;
        }

        public void setThesaurusIndex(int thesaurusIndex) {
            this.thesaurusIndex = thesaurusIndex;
        }

        public String getLibelleString() {
            return libelleString;
        }

        public void setLibelleString(String libelleString) {
            this.libelleString = libelleString;
        }

    }

}
