/* Scrutari - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.contentstreams;

import fr.exemole.sctserver.SctServer;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.scrutari.db.CountStats;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.db.stats.MotcleCountStats;
import net.mapeadores.scrutari.db.util.LibelleUtils;
import fr.exemole.sctserver.servlets.selection.SelectionUnit;
import net.mapeadores.opendocument.io.OdContentWriter;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.opendocument.io.ZipEntrySource;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.Libelle;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class MotsclesOdsStream implements ResponseHandler {

    private SctServer sctServer;
    private SelectionUnit selectionUnit;
    private int workingLangInteger;

    public MotsclesOdsStream(SctServer sctServer, LocalisationInfo localisationInfo, SelectionUnit selectionUnit) {
        this.sctServer = sctServer;
        this.selectionUnit = selectionUnit;
        this.workingLangInteger = localisationInfo.getWorkingLangInteger();
    }

    public long getLastModified() {
        return ResponseHandler.LASTMODIFIED_NOCACHE;
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType(OdUtils.getMimeType(OdUtils.OD_SPREADSHEET));
        OutputStream outputStream = response.getOutputStream();
        MotsclesOdsWriter lexieOdsWriter = new MotsclesOdsWriter();
        lexieOdsWriter.export(outputStream);
        outputStream.close();

    }


    private class MotsclesOdsWriter {

        private OdZip odZip;

        MotsclesOdsWriter() {
            odZip = new OdZip();
            odZip.setOdtype(OdUtils.OD_SPREADSHEET);
            odZip.setContentZipEntrySource(new ContentZipEntrySource());
        }

        public void export(OutputStream outputStream) throws IOException {
            odZip.zip(outputStream);
        }


        private class ContentZipEntrySource implements ZipEntrySource {

            String currentBaseLibelle;
            String currentThesaurusLibelle;

            public void writeEntry(OutputStream outputStream) throws IOException {
                BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
                MotsclesXMLWriter xmlWriter = new MotsclesXMLWriter();
                xmlWriter.setAppendable(writer);
                xmlWriter.setIndentLength(0);
                xmlWriter.init();
                xmlWriter.start();
                xmlWriter.printMotscles();
                xmlWriter.end();
                writer.flush();
            }


            private class MotsclesXMLWriter extends OdContentWriter {

                ScrutariDB scrutariDB;
                int[] motcleLangArray;
                DataAccess dataAccess;
                CountStats countStats;

                void init() {
                    scrutariDB = sctServer.getScrutariDB();
                    dataAccess = scrutariDB.openDataAccess();
                    motcleLangArray = scrutariDB.getLangStats().getAllMotcleLangArray();
                    countStats = scrutariDB.getCountStats();
                }

                private void start() throws IOException {
                    appendXMLDeclaration();
                    openDocumentContent();
                    openBody();
                    openSpreadsheet();
                    openTable("motscles");
                    printLibRow();

                }

                private void printLibRow() throws IOException {
                    openTableRow();

                    int length = motcleLangArray.length;
                    for (int i = 0; i < length; i++) {
                        addStringTableCell(LangInteger.toString(motcleLangArray[i]));
                    }
                    addStringTableCell("occurence");
                    addStringTableCell("thesaurus");
                    addStringTableCell("base");
                    closeTableRow();
                }

                private void printMotscles() throws IOException {
                    int baseCount = dataAccess.getBaseCount();
                    for (int i = 0; i < baseCount; i++) {
                        BaseData baseData = dataAccess.getBaseDataByIndex(i);
                        printBaseData(baseData);
                    }
                }

                private void printBaseData(BaseData baseData) throws IOException {
                    currentBaseLibelle = LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_SHORT, workingLangInteger, "?");
                    int thesaurusCount = baseData.getThesaurusCount();
                    for (int i = 0; i < thesaurusCount; i++) {
                        ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(baseData.getCodeThesaurus(i));
                        printThesaurusData(thesaurusData);
                    }
                }

                private void printThesaurusData(ThesaurusData thesaurusData) throws IOException {
                    if ((selectionUnit != null) && (!selectionUnit.contains(thesaurusData.getThesaurusURI()))) {
                        return;
                    }
                    currentThesaurusLibelle = LibelleUtils.getLibelleString(thesaurusData, workingLangInteger, "?");
                    int motcleCount = thesaurusData.getMotcleCount();
                    for (int i = 0; i < motcleCount; i++) {
                        printMotcleData(dataAccess.getMotcleDataByCode(thesaurusData.getCodeMotcle(i)));
                    }
                }

                private void printMotcleData(MotcleData motcleData) throws IOException {
                    openTableRow();
                    int length = motcleLangArray.length;
                    for (int i = 0; i < length; i++) {
                        Libelle libelle = motcleData.getLibelleByLang(motcleLangArray[i]);
                        if (libelle != null) {
                            addStringTableCell(libelle.getLibelleString());
                        } else {
                            addEmptyTableCell();
                        }
                    }
                    MotcleCountStats motcleCountStats = countStats.getMotcleCountStats(motcleData.getCodeMotcle());
                    addIntegerTableCell(motcleCountStats.getFicheCount());
                    addStringTableCell(currentThesaurusLibelle);
                    addStringTableCell(currentBaseLibelle);
                    closeTableRow();
                }

                private void end() throws IOException {
                    closeTable();
                    closeSpreadsheet();
                    closeBody();
                    closeDocumentContent();
                    dataAccess.close();
                }

            }

        }

    }

}
