/* Scrutari - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.servlets.admin.AdminConstants;
import fr.exemole.sctserver.servlets.admin.AdminRequest;
import fr.exemole.sctserver.servlets.resources.theme.SctCssCatalog;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.UtilHtmlPrinter;
import net.mapeadores.util.localisation.BundleLocalisation;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.servlets.ResponseHandler;
import java.io.IOException;
import java.util.Locale;
import java.util.ResourceBundle;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.mapeadores.util.localisation.LocalisationInfo;


/**
 *
 * @author Vincent Calame
 */
public abstract class AdminHtmlPage extends UtilHtmlPrinter implements SctCssCatalog, ResponseHandler, AdminConstants {

    private final static HTMLAttributes divAttr = new HTMLAttributes();
    private final static HTMLAttributes spanAttr = new HTMLAttributes();
    private final static HTMLAttributes aAttr = new HTMLAttributes();

    static {
        divAttr.setClasses(TOOL_DIVCLASS);
        spanAttr.setClasses(CURRENT_SPANCLASS);
        aAttr.setClasses(PAGE_ACLASS);
    }

    protected SctServer sctServer;
    protected int workingLangInteger;
    private CommandMessage commandMessage;
    private short currentPage;

    public AdminHtmlPage() {
        addCssUrl("../css/admin.css");
    }

    protected void init(SctServer sctServer) {
        this.sctServer = sctServer;
    }

    public void setLocalisationInfo(LocalisationInfo localisationInfo) {
        this.workingLangInteger = localisationInfo.getWorkingLangInteger();
        Locale locale = localisationInfo.getFormatLocale();
        ResourceLocalisation textResourceLocalisation = new BundleLocalisation(ResourceBundle.getBundle("fr.exemole.sctserver.servlets.resources.l10n.sctadmin", locale));
        addResourceLocalisation(textResourceLocalisation);
        ResourceLocalisation commandResourceLocalisation = new BundleLocalisation(ResourceBundle.getBundle("fr.exemole.sctserver.servlets.resources.l10n.sctfb", locale));
        setCommandKeyLocalisation(commandResourceLocalisation);
        
    }

    public void setCurrentPage(short currentPage) {
        this.currentPage = currentPage;
    }


    public void setCommandMessage(CommandMessage commandMessage) {
        this.commandMessage = commandMessage;
    }

    public long getLastModified() {
        return ResponseHandler.LASTMODIFIED_NOCACHE;
    }

    protected void initResponse(HttpServletResponse response) throws IOException {
        response.setContentType("text/html;charset=UTF-8");
        initPrinter(response.getWriter());

    }

    protected void printTools() {
        startDiv(divAttr);
        printToolLink(INDEX_PAGE);
        printToolSeparator();
        printToolLink(STATS_PAGE);
        printToolSeparator();
        printToolLink(FICHES_PAGE);
        printToolSeparator();
        printToolLink(MOTSCLES_PAGE);
        printToolSeparator();
        printToolLink(LEXIES_PAGE);
        printToolSeparator();
        printToolLink(JOURNAUX_PAGE);
        endDiv();

    }

    protected void printMessage() {
        if (commandMessage != null) {
            HtmlPrinterUtils.printCommandMessage(this, commandMessage, DONEMESSAGE_SPANCLASS, ERRORMESSAGE_SPANCLASS);
        }
    }

    protected void start() {
        start(workingLangInteger);
    }

    private void printToolSeparator() {
        append(" — ");
    }

    private void printToolLink(short page) {
        String key = AdminRequest.pageToString(page);
        if (currentPage == page) {
            startSpan(spanAttr);
        } else {
            HTMLAttributes aCopy = aAttr.copy();
            aCopy.setHref(key + ".html");
            startA(aCopy);
        }
        localize("page_" + key);
        if (currentPage == page) {
            endSpan();
        } else {
            endA();
        }
    }

    public SctServer getSctServer() {
        return sctServer;
    }

}
