/* Scrutari - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.ConfState;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.conf.ConfConstants;
import fr.exemole.sctserver.servlets.admin.ConfUpdateCommand;
import fr.exemole.sctserver.servlets.admin.RestartCommand;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.CorpusClassesPrinter;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.SearchLogFilterPrinter;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.SourcesPrinter;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.UriCodesPrinter;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.HTMLAttributes;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.CommandBox;


/**
 *
 * @author Vincent Calame
 */
public class IndexPage extends AdminHtmlPage implements ConfConstants {

    private ConfState confState;

    public IndexPage(SctServer sctServer) {
        init(sctServer);
        confState = sctServer.getConfState();
        HTMLAttributes formAttributes = new HTMLAttributes();
        formAttributes.setAction(".");
        formAttributes.setMethod("get");
        getCommandBoxParameters().setFormAttributes(formAttributes);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTools();
        printMessage();

        if (sctServer.hasUpdateProcessLog()) {
            startP();
            startSpan(ERRORMESSAGE_SPANCLASS);
            localize("info_updateprocesslog");
            space();
            startA("../logs/log_updateprocess.txt");
            append("log_updateprocess.txt");
            endA();
            endSpan();
            endP();
        }

        printRestartForm();

        printConfState();

        end();
    }

    private void printConfState() {
        appendH2("title_conf_files");
        short state = confState.getGlobalState();
        boolean printConfDetails = false;
        switch (state) {
            case ConfState.GLOBAL_CONFIO_ERROR:
                startP();
                startSpan(ERRORMESSAGE_SPANCLASS);
                localize("error_global_confio");
                append(" : ");
                escape(confState.getConfIOMessage());
                endSpan();
                endP();
                break;
            case ConfState.GLOBAL_UNDEFINED:
                startP();
                startSpan(ERRORMESSAGE_SPANCLASS);
                localize("error_global_confnull");
                append(" : ");
                escape(confState.getConfIOMessage());
                endSpan();
                endP();
                break;
            case ConfState.GLOBAL_OK:
                startUl();
                printConfFileLi(SOURCES_NAME);
                printConfFileLi(CORPUS_CLASSES_NAME);
                printConfFileLi(SEARCHLOG_FILTER_NAME);
                printConfFileLi(URI_CODES_NAME);
                printConfFileLi(FEED_NAME);
                endUl();
                printConfDetails = true;
                break;
        }
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction(".");
        commandBox.setCommandFamily("SCT");
        commandBox.setCommandString(ConfUpdateCommand.COMMANDNAME);
        commandBox.setCommandKey(ConfUpdateCommand.COMMANDKEY);
        startCommandBox(commandBox);
        endCommandBox("submit_update");
        if (printConfDetails) {
            appendH2("title_conf_details");
            printSourcesDetails();
            printCorpusClassesDetails();
            printSearchLogFilterDetails();
            printUriCodesDetails();
            printFeedDetails();
        }
    }

    private void printConfFileLi(String name) {
        startLi();
        printFileInfo(name, true);
        endLi();
    }

    private void printSourcesDetails() {
        HTMLAttributes h3Attributes = new HTMLAttributes();
        h3Attributes.setId(SOURCES_NAME);
        startH3(h3Attributes);
        localize("title_conf_sources");
        append(" / ");
        printFileInfo(SOURCES_NAME, false);
        endH3();
        printLog(SOURCES_NAME);
        SourcesPrinter.printSourceInfoList(this);
    }

    private void printCorpusClassesDetails() {
        HTMLAttributes h3Attributes = new HTMLAttributes();
        h3Attributes.setId(CORPUS_CLASSES_NAME);
        startH3(h3Attributes);
        localize("title_conf_corpusclasses");
        append(" / ");
        printFileInfo(CORPUS_CLASSES_NAME, false);
        endH3();
        printLog(CORPUS_CLASSES_NAME);
        CorpusClassesPrinter.printCorpusClasses(this);
    }

    private void printSearchLogFilterDetails() {
        HTMLAttributes h3Attributes = new HTMLAttributes();
        h3Attributes.setId(SEARCHLOG_FILTER_NAME);
        startH3(h3Attributes);
        localize("title_conf_searchlogfilter");
        append(" / ");
        printFileInfo(SEARCHLOG_FILTER_NAME, false);
        endH3();
        printLog(SEARCHLOG_FILTER_NAME);
        SearchLogFilterPrinter.printSearchLogFilter(this);
    }

    private void printUriCodesDetails() {
        HTMLAttributes h3Attributes = new HTMLAttributes();
        h3Attributes.setId(URI_CODES_NAME);
        startH3(h3Attributes);
        localize("title_conf_uricodes");
        append(" / ");
        printFileInfo(URI_CODES_NAME, false);
        endH3();
        printLog(URI_CODES_NAME);
        UriCodesPrinter.printUriCodes(this);
    }

    private void printFeedDetails() {
        HTMLAttributes h3Attributes = new HTMLAttributes();
        h3Attributes.setId(FEED_NAME);
        startH3(h3Attributes);
        localize("title_conf_feed");
        append(" / ");
        printFileInfo(FEED_NAME, false);
        endH3();
        printLog(FEED_NAME);
    }

    private void printRestartForm() throws IOException {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setName(RestartCommand.FORCE_PARAMNAME);
        htmlAttributes.setValue("1");
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction(".");
        commandBox.setCommandFamily("SCT");
        commandBox.setCommandString(RestartCommand.COMMANDNAME);
        commandBox.setCommandKey(RestartCommand.COMMANDKEY);
        startCommandBox(commandBox);
        startP();
        addCheck(htmlAttributes, "lib_restart_force", null);
        endP();
        endCommandBox("submit_restart");
    }

    private void printLog(String name) {
        short fileState = confState.getFileState(name);
        switch (fileState) {
            case FILE_STATE_UNKNOWN:
            case FILE_MISSING:
            case FILE_OK:
                return;
        }
        startP();
        localize("title_log");
        endP();
        HTMLAttributes preAttributes = new HTMLAttributes();
        preAttributes.addClass(LOG_PRECLASS);
        startPre(preAttributes);
        escape(sctServer.getLog(name), true);
        endPre();
    }

    private void printFileInfo(String name, boolean withLink) {
        String[] array = getKeys(confState.getFileState(name));
        HTMLAttributes spanAttributes = new HTMLAttributes();
        spanAttributes.addClass(array[1]);
        if (withLink) {
            startA("#" + name);
        }
        append(name);
        append(".xml");
        if (withLink) {
            endA();
        }
        append(" (");
        startSpan(spanAttributes);
        localize("file_" + array[0]);
        endSpan();
        append(")");
    }

    private String[] getKeys(short fileState) {
        String spanClass = null;
        String key = null;
        switch (fileState) {
            case FILE_STATE_UNKNOWN:
                key = "state_unknown";
                spanClass = NEUTRAL_SPANCLASS;
                break;
            case FILE_MISSING:
                key = "missing";
                spanClass = NEUTRAL_SPANCLASS;
                break;
            case FILE_XML_MALFORMED:
                key = "xml_malformed";
                spanClass = ERROR_SPANCLASS;
                break;
            case FILE_XML_ERROR:
                key = "xml_error";
                spanClass = ERROR_SPANCLASS;
                break;
            case FILE_XML_WARNING:
                key = "xml_warning";
                spanClass = WARNING_SPANCLASS;
                break;
            case FILE_OK:
                key = "ok";
                spanClass = OK_SPANCLASS;
                break;
            default:
                throw new SwitchException("Unknown getFileState : " + fileState);
        }
        String[] array = new String[2];
        array[0] = key;
        array[1] = spanClass;
        return array;
    }

}
