/* Scrutari - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.SearchLog;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.primitives.DateInteger;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;


/**
 *
 * @author Vincent Calame
 */
public class JournalPage extends AdminHtmlPage {

    private int dateInteger;
    private HTMLAttributes tableAttributes;

    public JournalPage(SctServer sctServer, int dateInteger) {
        init(sctServer);
        this.dateInteger = dateInteger;
        tableAttributes = new HTMLAttributes();
        tableAttributes.addClass(TABLE_STATS);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTools();
        startH1();
        append(DateInteger.toString(dateInteger));
        endH1();
        SearchLog.DayLog[] dayLogArray = sctServer.getSearchLog().getDayLogArray(dateInteger);
        for (int i = 0; i < dayLogArray.length; i++) {
            printDayLog(dayLogArray[i]);
        }
        end();
    }

    private void printDayLog(SearchLog.DayLog dayLog) {
        SearchLog.DayStats dayStats = dayLog.getDayStats();
        startH2();
        append(String.valueOf(DateInteger.getJour(dayLog.getDateInteger())));
        space();
        startSmall();
        append("(");
        appendInt(dayStats.getFicheCount());
        append(" / ");
        appendInt(dayStats.getMotcleCount());
        append(")");
        endSmall();
        endH2();
        startTable(tableAttributes);
        SearchLog.QueryLog[] queryLogArray = dayLog.getQueryLogArray();
        for (int i = 0; i < queryLogArray.length; i++) {
            printQueryLog(queryLogArray[i]);
        }
        endTable();
    }

    private void printQueryLog(SearchLog.QueryLog queryLog) {
        startRow();
        startCell();
        append(queryLog.getRemoteAddress());
        endCell();
        startCell();
        escape(queryLog.getSearchSequence());
        endCell();
        startCell();
        append(queryLog.getSite());
        endCell();
        startCell();
        append(queryLog.getReferrer());
        endCell();
        startCell();
        appendInt(queryLog.getFicheCount());
        endCell();
        startCell();
        appendInt(queryLog.getFicheMaximum());
        endCell();
        startCell();
        appendInt(queryLog.getCompleteMotcleCount());
        endCell();
        startCell();
        appendInt(queryLog.getAllMotcleCount());
        endCell();
        endRow();
    }

    private void appendInt(int val) {
        if (val < 0) {
            return;
        }
        append(String.valueOf(val));
    }

}
