/* Scrutari - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.servlets.selection.SelectionManager;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.db.util.LibelleUtils;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.Libelle;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;


/**
 *
 * @author Vincent Calame
 */
public class MotsclesPage extends AdminHtmlPage {

    private HTMLAttributes theadAttributes = new HTMLAttributes();
    private HTMLAttributes baseTBodyAttributes = new HTMLAttributes();
    private HTMLAttributes baseCellAttributes = new HTMLAttributes();
    private HTMLAttributes thesaurusTBodyAttributes = new HTMLAttributes();
    private HTMLAttributes thesaurusCellAttributes = new HTMLAttributes();
    private HTMLAttributes motcleTBodyAttributes = new HTMLAttributes();

    {
        thesaurusCellAttributes.setClasses("c_SCT_lst_thesaurus");
        motcleTBodyAttributes.setStyle("display: none;");
    }

    private ScrutariDB scrutariDB;
    private int[] motcleLangArray;
    private SelectionManager selectionManager;

    public MotsclesPage(SctServer sctServer, SelectionManager selectionManager) {
        init(sctServer);
        this.selectionManager = selectionManager;
        scrutariDB = sctServer.getScrutariDB();
        addCssUrl("../css/listes.css");
        addJsUrl("../js/affichage.js");
        motcleLangArray = scrutariDB.getLangStats().getAllMotcleLangArray();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        initResponse(response);
        start();
        printTools();
        printMessage();
        startP();
        append("ods : ");
        startA("motscles.ods");
        append("_all");
        endA();
        for (String name : selectionManager.getNameSet()) {
            printLien(name);
        }
        endP();
        startTable();
        startTHead(theadAttributes);
        startRow();
        startCell();
        append("&nbsp;");
        endCell();
        startCell();
        append("&nbsp;");
        endCell();

        int length = motcleLangArray.length;
        for (int i = 0; i < length; i++) {
            startCell();
            append(LangInteger.toString(motcleLangArray[i]));
            endCell();
        }
        endRow();
        endTHead();

        baseCellAttributes.setColSpan(2 + length);
        thesaurusCellAttributes.setColSpan(1 + length);

        int baseCount = dataAccess.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            printBaseData(baseData, dataAccess);
        }
        endTable();
        end();
        dataAccess.close();
    }

    private void printLien(String name) {
        append(" - ");
        startA("motscles-" + name + ".ods");
        append(name);
        endA();
    }

    private void printBaseData(BaseData baseData, DataAccess dataAccess) {
        int codebase = baseData.getCodeBase();
        int thesaurusCount = baseData.getThesaurusCount();
        startTBody(baseTBodyAttributes);
        startRow();
        startCell(baseCellAttributes);
        escape(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_LONG, workingLangInteger, "?"));
        append(" ");
        startCode();
        append("[");
        append(String.valueOf(thesaurusCount));
        append("]");
        endCode();
        endCell();
        endRow();
        endTBody();
        for (int i = 0; i < thesaurusCount; i++) {
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(baseData.getCodeThesaurus(i));
            printThesaurusData(thesaurusData, dataAccess);
        }
    }

    private void printThesaurusData(ThesaurusData thesaurusData, DataAccess dataAccess) {
        int codethesaurus = thesaurusData.getCodeThesaurus();
        int motcleCount = thesaurusData.getMotcleCount();
        String onClick = "afficheRowGroup('tbody_" + codethesaurus + "');";
        thesaurusCellAttributes.setOnClick(onClick);
        startTBody(thesaurusTBodyAttributes);
        startRow();
        appendEmptyCell();
        startCell(thesaurusCellAttributes);
        escape(LibelleUtils.getLibelleString(thesaurusData, workingLangInteger, "?"));
        append(" ");
        startCode();
        append("[");
        append(String.valueOf(motcleCount));
        append("]");
        endCode();
        endCell();

        endRow();
        endTBody();

        motcleTBodyAttributes.setId("tbody_" + codethesaurus);
        startTBody(motcleTBodyAttributes);
        for (int i = 0; i < motcleCount; i++) {
            printMotcleData(dataAccess.getMotcleDataByCode(thesaurusData.getCodeMotcle(i)));
        }
        endTBody();

    }

    private void printMotcleData(MotcleData motcleData) {
        startRow();
        appendEmptyCell();
        appendEmptyCell();
        int length = motcleLangArray.length;
        for (int i = 0; i < length; i++) {
            startCell();
            Libelle libelle = motcleData.getLibelleByLang(motcleLangArray[i]);
            if (libelle != null) {
                escape(libelle.getLibelleString());
            }
            endCell();
        }
        endRow();
    }

}
