/* Scrutari - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.explore.xml;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.SearchLog;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class LogXMLPart extends XMLPart {

    private SctServer sctServer;

    public LogXMLPart(XMLWriter xmlWriter, SctServer sctServer) {
        super(xmlWriter);
        this.sctServer = sctServer;
    }

    public void appendLogList() throws IOException {
        openTag("log-list");
        int[] moisArray = sctServer.getSearchLog().getMonthIntegerArray();
        for (int i = 0; i < moisArray.length; i++) {
            int mois = moisArray[i];
            startLog(mois);
            closeEmptyTag();
        }
        closeTag("log-list");
    }

    private void startLog(int mois) throws IOException {
        startOpenTag("log");
        int annee = DateInteger.getAnnee(mois);
        addAttribute("y", String.valueOf(annee));
        int m = DateInteger.getMois(mois);
        String s = String.valueOf(m);
        if (m < 10) {
            s = "0" + s;
        }
        addAttribute("m", s);
        addAttribute("c", DateInteger.toString(mois));
    }

    public void appendLog(int mois) throws IOException {
        startLog(mois);
        endOpenTag();
        SearchLog.DayLog[] dayLogArray = sctServer.getSearchLog().getDayLogArray(mois);
        for (int i = 0; i < dayLogArray.length; i++) {
            appendDayLog(dayLogArray[i]);
        }
        closeTag("log");
    }

    public void appendDayLog(SearchLog.DayLog dayLog) throws IOException {
        IpMap ipMap = new IpMap();
        int jour = DateInteger.getJour(dayLog.getDateInteger());
        String n = String.valueOf(jour);
        String s = (jour < 10) ? "0" + n : n;
        SearchLog.QueryLog[] queryLogArray = dayLog.getQueryLogArray();
        for (int i = 0; i < queryLogArray.length; i++) {
            ipMap.addQueryLog(queryLogArray[i]);
        }
        startOpenTag("day");
        addAttribute("n", n);
        addAttribute("s", s);
        SearchLog.DayStats dayStats = dayLog.getDayStats();
        addIntAttribute("fiche", dayStats.getFicheCount());
        addIntAttribute("motcle", dayStats.getMotcleCount());
        addIntAttribute("corpus", dayStats.getCorpusCount());
        addIntAttribute("thesaurus", dayStats.getThesaurusCount());
        addIntAttribute("base", dayStats.getBaseCount());
        addIntAttribute("lexie", dayStats.getLexieCount());
        addIntAttribute("indexation", dayStats.getIndexationCount());
        endOpenTag();
        ipMap.append();
        closeTag("day");
    }

    private void appendQueryLog(SearchLog.QueryLog queryLog) throws IOException {
        startOpenTag("q");
        addAttribute("seq", queryLog.getSearchSequence());
        addAttribute("site", queryLog.getSite());
        addAttribute("ref", queryLog.getReferrer());
        addIntAttribute("fc", queryLog.getFicheCount());
        addIntAttribute("fm", queryLog.getFicheMaximum());
        addIntAttribute("cmc", queryLog.getCompleteMotcleCount());
        addIntAttribute("amc", queryLog.getAllMotcleCount());
        closeEmptyTag();
    }

    private void addIntAttribute(String name, int val) throws IOException {
        if (val < 0) {
            return;
        }
        addAttribute(name, val);
    }


    private class IpMap {

        Map<String, List<SearchLog.QueryLog>> map = new LinkedHashMap<String, List<SearchLog.QueryLog>>();

        void addQueryLog(SearchLog.QueryLog queryLog) {
            String ip = queryLog.getRemoteAddress();
            List<SearchLog.QueryLog> list = map.get(ip);
            if (list == null) {
                list = new ArrayList<SearchLog.QueryLog>();
                map.put(ip, list);
            }
            list.add(queryLog);
        }

        void append() throws IOException {
            for (List<SearchLog.QueryLog> list : map.values()) {
                openTag("ip");
                for (SearchLog.QueryLog queryLog : list) {
                    appendQueryLog(queryLog);
                }
                closeTag("ip");
            }
        }

    }

}
