/* Scrutari - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.feed;

import fr.exemole.sctserver.SctContext;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class FeedRequest implements FeedConstants {

    private final static String ATOM_EXTENSION = ".atom";
    private boolean isInit;
    private short responseHandlerType;
    private SctContext sctContext;
    private LocalisationInfo localisationInfo;
    private FeedOptions feedOptions;

    private FeedRequest(SctContext sctContext, String pathInfo) {
        this.sctContext = sctContext;
        if (!pathInfo.endsWith(ATOM_EXTENSION)) {
            return;
        }
        String base = pathInfo.substring(0, pathInfo.length() - ATOM_EXTENSION.length());
        int idx = base.lastIndexOf('_');
        if (idx == -1) {
            return;
        }
        try {
            int langInteger = LangInteger.parse(base.substring(idx+1));
            localisationInfo = new LocalisationInfo(langInteger);
        } catch (LangIntegerException lie) {
            return;
        }
        String options = null;
        if (base.startsWith("/tree")) {
            options = base.substring(5, idx);
        } else if (base.startsWith("/data")) {
            options = base.substring(5, idx);
        } else {
            return;
        }
        feedOptions = FeedOptions.parse(options);
        isInit = true;
        responseHandlerType = ATOM_FEED;
    }

    public static FeedRequest init(SctContext sctContext, String pathInfo) {
        FeedRequest feedRequest = new FeedRequest(sctContext, pathInfo);
        if (feedRequest.isInit) {
            return feedRequest;
        } else {
            return null;
        }
    }

    public ResponseHandler getResponseHandler() {
        switch (responseHandlerType) {
            case ATOM_FEED:
                return new UriTreeAtom(sctContext, localisationInfo, feedOptions);
            default:
                throw new SwitchException("responseHandlerType = " + responseHandlerType);
        }
    }

}
