/* ScrutariDataURI - Copyright (c) 2005-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.datauri;


/**
 *
 * @author  Vincent Calame
 */
public final class ThesaurusURI extends ScrutariDataURI implements Comparable<ThesaurusURI> {

    private BaseURI baseURI;
    private String thesaurusName;

    public ThesaurusURI(String authorityUUID, String baseName, String thesaurusName) {
        this.thesaurusName = thesaurusName;
        this.baseURI = new BaseURI(authorityUUID, baseName);
    }

    public ThesaurusURI(BaseURI baseURI, String thesaurusName) {
        this.thesaurusName = thesaurusName;
        this.baseURI = baseURI;
    }

    public BaseURI getBaseURI() {
        return baseURI;
    }

    public String getThesaurusName() {
        return thesaurusName;
    }

    @Override
    public int hashCode() {
        return thesaurusName.hashCode() + baseURI.hashCode();
    }

    public int compareTo(ThesaurusURI thesaurusURI) {
        int result = this.thesaurusName.compareTo(thesaurusURI.thesaurusName);
        if (result == 0) {
            result = this.baseURI.compareTo(thesaurusURI.baseURI);
        }
        return result;
    }

    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof ThesaurusURI)) {
            return false;
        }
        return (compareTo((ThesaurusURI) o) == 0);
    }

    public short getType() {
        return THESAURUSURI_TYPE;
    }

    protected void appendPath(StringBuilder buf) {
        buf.append('/');
        buf.append(baseURI.getAuthorityUUID());
        buf.append('/');
        buf.append(baseURI.getBaseName());
        buf.append('/');
        buf.append(thesaurusName);
    }

}
