/* ScrutariLib_DB - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.db.lexiebuilder;

import net.mapeadores.scrutari.lexie.Lexification;
import net.mapeadores.util.localisation.LocalisationLexieFilters;
import net.mapeadores.util.text.collation.CollationUnitPosition;
import net.mapeadores.util.text.lexie.LexieParser;
import net.mapeadores.util.text.lexie.LexieFilter;
import net.mapeadores.util.text.lexie.LexieParserListener;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.lexie.FicheLexification;
import net.mapeadores.scrutari.lexie.FieldNumbers;
import net.mapeadores.scrutari.lexie.LexieOccurrences;
import net.mapeadores.scrutari.lexie.LexieOccurrencesList;
import net.mapeadores.scrutari.lexie.MotcleLexification;
import net.mapeadores.util.text.Libelle;


/**
 *
 * @author Vincent Calame
 */
public class LexieOccurrencesMapBuilder {

    private LexieUnitBuilder lexieUnitBuilder;
    private InternalLexieParserListener listener = new InternalLexieParserListener();
    private int currentSubsetCode;
    private int currentLangInteger;
    private int currentFieldNumber;
    private int currentCode;
    private String currentSource;
    private int currentWholeLength;
    private List<Long> currentLexieIdList = new ArrayList<Long>();
    private List<CollationUnitPosition> currentCollationUnitPositionList = new ArrayList<CollationUnitPosition>();
    private LexieFilter currentLexieFilter;
    private Map<Long, InternalLexieOccurencesList> lexieOccurencesListMap = new HashMap<Long, InternalLexieOccurencesList>();
    private Map<Long, LexieOccurrencesBuilder> currentLexieOccurrencesBuilderMap = new HashMap<Long, LexieOccurrencesBuilder>();
    private boolean onFiche = false;

    public LexieOccurrencesMapBuilder(LexieUnitBuilder lexieUnitBuilder, boolean onFiche) {
        this.lexieUnitBuilder = lexieUnitBuilder;
        this.onFiche = onFiche;
    }

    public int getCurrentSubsetCode() {
        return currentSubsetCode;
    }

    public void setCurrentSubsetCode(int currentSubsetCode) {
        this.currentSubsetCode = currentSubsetCode;
    }

    public Map<Long, LexieOccurrencesList> popLexieOccurencesListMap() {
        Map<Long, LexieOccurrencesList> result = new HashMap<Long, LexieOccurrencesList>(lexieOccurencesListMap);
        lexieOccurencesListMap.clear();
        return result;
    }

    public FicheLexification initFicheData(FicheData ficheData) {
        setCurrentCode(ficheData.getCodeFiche());
        setCurrentLangInteger(ficheData.getLangInteger());
        FicheLexification ficheLexification = new FicheLexification();

        setCurrentFieldNumber(FieldNumbers.TITRE_FIELDNUMBER);
        tokenize(ficheData.getTitre());
        flushLexification(ficheLexification);

        setCurrentFieldNumber(FieldNumbers.SOUSTITRE_FIELDNUMBER);
        tokenize(ficheData.getSoustitre());
        flushLexification(ficheLexification);

        for (int i = 1; i <= ficheData.getComplementMaxNumber(); i++) {
            String complement = ficheData.getComplementByNumber(i);
            setCurrentFieldNumber(i);
            tokenize(complement);
            flushLexification(ficheLexification);
        }

        flushLexieOccurrencesBuilderMap();
        return ficheLexification;
    }

    public MotcleLexification initMotcleData(MotcleData motcleData) {
        setCurrentCode(motcleData.getCodeMotcle());
        setCurrentFieldNumber(FieldNumbers.NONE_FIELDNUMBER);
        MotcleLexification motcleLexification = new MotcleLexification();
        int libelleCount = motcleData.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            Libelle libelle = motcleData.getLibelleByIndex(i);
            setCurrentLangInteger(libelle.getLangInteger());
            tokenize(libelle.getLibelleString());
            flushLexification(motcleLexification);
        }
        flushLexieOccurrencesBuilderMap();
        return motcleLexification;
    }

    private void tokenize(String chaine) {
        if (chaine == null) {
            return;
        }
        int length = chaine.length();
        if (length == 0) {
            return;
        }
        setCurrentSource(chaine);
        setCurrentWholeLength(length);
        LexieParser.parse(chaine, listener);
    }

    private void addLexieOccurrence(long lexieId, CollationUnitPosition collationUnitPosition) {
        LexieOccurrencesBuilder lexieOccurrencesBuilder = currentLexieOccurrencesBuilderMap.get(lexieId);
        if (lexieOccurrencesBuilder == null) {
            lexieOccurrencesBuilder = new LexieOccurrencesBuilder(lexieId);
            currentLexieOccurrencesBuilderMap.put(lexieId, lexieOccurrencesBuilder);
        }
        int lexieIndexInField = currentLexieIdList.size();
        currentLexieIdList.add(lexieId);
        currentCollationUnitPositionList.add(collationUnitPosition);
        lexieOccurrencesBuilder.addOccurrence(lexieIndexInField, collationUnitPosition);
    }

    private void flushLexieOccurrencesBuilderMap() {
        for (LexieOccurrencesBuilder lexieOccurrencesBuilder : currentLexieOccurrencesBuilderMap.values()) {
            long lexieId = lexieOccurrencesBuilder.lexieId;
            InternalLexieOccurencesList lexieOccurencesArrayBuilder = lexieOccurencesListMap.get(lexieId);
            if (lexieOccurencesArrayBuilder == null) {
                lexieOccurencesArrayBuilder = new InternalLexieOccurencesList();
                lexieOccurencesListMap.put(lexieId, lexieOccurencesArrayBuilder);
            }
            lexieOccurencesArrayBuilder.addLexieOccurrences(lexieOccurrencesBuilder.toLexieOccurences());
        }
        currentLexieOccurrencesBuilderMap.clear();
    }

    private Lexification flushLexification() {
        int size = currentLexieIdList.size();
        if (size == 0) {
            return null;
        }
        long[] lexieIdArray = new long[size];
        CollationUnitPosition[] collationUnitPositionArray = new CollationUnitPosition[size];
        for (int i = 0; i < size; i++) {
            lexieIdArray[i] = currentLexieIdList.get(i);
            collationUnitPositionArray[i] = currentCollationUnitPositionList.get(i);
        }
        Lexification lexification = new Lexification();
        lexification.setWholeLength(getCurrentWholeLength());
        lexification.setLexieIdArray(lexieIdArray);
        lexification.setCollationUnitPositionArray(collationUnitPositionArray);
        currentLexieIdList.clear();
        currentCollationUnitPositionList.clear();
        return lexification;
    }

    private int getCurrentLangInteger() {
        return currentLangInteger;
    }

    private void setCurrentLangInteger(int currentLangInteger) {
        this.currentLangInteger = currentLangInteger;
        currentLexieFilter = LocalisationLexieFilters.getLexieFilter(currentLangInteger);
    }

    private int getCurrentFieldNumber() {
        return currentFieldNumber;
    }

    private void setCurrentFieldNumber(int currentFieldNumber) {
        this.currentFieldNumber = currentFieldNumber;
    }

    private int getCurrentCode() {
        return currentCode;
    }

    private void setCurrentCode(int currentCode) {
        this.currentCode = currentCode;
    }

    private void setCurrentSource(String currentSource) {
        this.currentSource = currentSource;
    }

    private int getCurrentWholeLength() {
        return currentWholeLength;
    }

    private void setCurrentWholeLength(int currentWholeLength) {
        this.currentWholeLength = currentWholeLength;
    }

    private void flushLexification(FicheLexification ficheLexification) {
        Lexification lexification = flushLexification();
        if (lexification != null) {
            ficheLexification.putLexification(getCurrentFieldNumber(), lexification);
        }
    }

    private void flushLexification(MotcleLexification motcleLexification) {
        Lexification lexification = flushLexification();
        if (lexification != null) {
            motcleLexification.putLexification(getCurrentLangInteger(), lexification);
        }
    }


    private class InternalLexieOccurencesList implements LexieOccurrencesList {

        private List<LexieOccurrences> lexieOccurrencesList = new ArrayList<LexieOccurrences>();

        private InternalLexieOccurencesList() {
        }

        public int getLexieOccurrencesCount() {
            return lexieOccurrencesList.size();
        }

        public LexieOccurrences getLexieOccurrences(int i) {
            return lexieOccurrencesList.get(i);
        }

        private void addLexieOccurrences(LexieOccurrences lexieOccurrences) {
            lexieOccurrencesList.add(lexieOccurrences);
        }

    }


    private class LexieOccurrencesBuilder {

        private long lexieId;
        private List<OccurrenceByField> occurenceByFieldList = new ArrayList<OccurrenceByField>();
        private OccurrenceByField lastOccurrenceByField;

        private LexieOccurrencesBuilder(long lexieId) {
            this.lexieId = lexieId;
        }

        protected void addOccurrence(int lexieIndexInField, CollationUnitPosition collationUnitPosition) {
            if ((lastOccurrenceByField == null) || (lastOccurrenceByField.fieldNumber != getCurrentFieldNumber())) {
                lastOccurrenceByField = new OccurrenceByField(collationUnitPosition);
                occurenceByFieldList.add(lastOccurrenceByField);
            }
            lastOccurrenceByField.addLexieIndexInLexification(lexieIndexInField);
        }

        public LexieOccurrences toLexieOccurences() {
            LexieOccurrences lexieOccurrences = new LexieOccurrences();
            lexieOccurrences.setCode(getCurrentCode());
            int length = occurenceByFieldList.size();
            int[] fieldNumberArray = new int[length];
            CollationUnitPosition[] bestPositionInFieldArray = new CollationUnitPosition[length];
            int[] fieldWholeLengthArray = new int[length];
            int[][] lexieIndexInLexificationArray = new int[length][];
            for (int i = 0; i < length; i++) {
                OccurrenceByField occurrenceByField = occurenceByFieldList.get(i);
                fieldNumberArray[i] = occurrenceByField.fieldNumber;
                bestPositionInFieldArray[i] = occurrenceByField.bestPositionInfield;
                fieldWholeLengthArray[i] = occurrenceByField.fieldWholeLength;
                lexieIndexInLexificationArray[i] = occurrenceByField.getLexieIndexInLexificationArray();
            }
            lexieOccurrences.setArrays(fieldNumberArray, bestPositionInFieldArray, fieldWholeLengthArray, lexieIndexInLexificationArray);
            return lexieOccurrences;
        }


        class OccurrenceByField {

            private int fieldNumber;
            private CollationUnitPosition bestPositionInfield;
            private int fieldWholeLength;
            private List<Integer> lexieIndexInLexificationList = new ArrayList<Integer>();

            OccurrenceByField(CollationUnitPosition bestPositionInfield) {
                this.fieldNumber = getCurrentFieldNumber();
                this.bestPositionInfield = bestPositionInfield;
                fieldWholeLength = getCurrentWholeLength();
            }

            private void addLexieIndexInLexification(int lexieIndexInLexification) {
                lexieIndexInLexificationList.add(lexieIndexInLexification);
            }

            private int[] getLexieIndexInLexificationArray() {
                int size = lexieIndexInLexificationList.size();
                int[] result = new int[size];
                for (int i = 0; i < size; i++) {
                    result[i] = lexieIndexInLexificationList.get(i);
                }
                return result;
            }

        }

    }


    private class InternalLexieParserListener implements LexieParserListener {

        InternalLexieParserListener() {
        }

        public void flushLexie(String lexie, int lexieStartIndex) {
            if (currentLexieFilter.acceptLexie(lexie)) {
                LexieOccurence lexieOccurence = lexieUnitBuilder.getLexieOccurence(getCurrentLangInteger(), lexie, onFiche);
                CollationUnitPosition collationUnitPosition = new CollationUnitPosition(lexieOccurence.getCollationUnit(), lexieStartIndex);
                addLexieOccurrence(lexieOccurence.getLexieId(), collationUnitPosition);
            }
        }

        public void checkSpecialChar(char carac) {
        }

    }

}
