/* ScrutariLib_Data - Copyright (c) 2005-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.data;

import net.scrutari.datauri.ThesaurusURI;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.collections.LinkedIntHashMap;
import java.util.List;
import java.util.ArrayList;
import java.io.Serializable;


/**
 *
 * @author Vincent Calame
 */
public final class ThesaurusData implements Serializable {

    private int codethesaurus;
    private int codebase;
    private LinkedIntHashMap intLangMap;
    private int[] codesMotscles;
    private ThesaurusURI thesaurusURI;

    public ThesaurusData(ThesaurusURI thesaurusURI, int codethesaurus, int codebase, int[] codesMotscles, LinkedIntHashMap intLangMap) {
        this.codethesaurus = codethesaurus;
        this.codebase = codebase;
        this.intLangMap = intLangMap;
        this.codesMotscles = codesMotscles;
        this.thesaurusURI = thesaurusURI;
    }

    public int getCodeThesaurus() {
        return codethesaurus;
    }

    public int getCodeBase() {
        return codebase;
    }

    public ThesaurusURI getThesaurusURI() {
        return thesaurusURI;
    }

    public int getMotcleCount() {
        return codesMotscles.length;
    }

    public int getCodeMotcle(int i) {
        return codesMotscles[i];
    }

    public ThesaurusLangData getThesaurusLangDataByLang(int langInteger) {
        return (ThesaurusLangData) intLangMap.get(langInteger);
    }

    /**
     * Retourne null ou une chaîne de longueur non nulle et non composée d'espaces blancs.
     */
    public String getLibelleStringByLang(int langInteger) {
        ThesaurusLangData thesaurusLangInfo = getThesaurusLangDataByLang(langInteger);
        if (thesaurusLangInfo == null) {
            return null;
        }
        Libelle lib = thesaurusLangInfo.getLibelle();
        if (lib == null) {
            return null;
        }
        return lib.getLibelleString();
    }

    public int getThesaurusLangDataCount() {
        return intLangMap.size();
    }

    public ThesaurusLangData getThesaurusLangDataByIndex(int index) {
        return (ThesaurusLangData) intLangMap.getValueAt(index);
    }


    public static class ThesaurusLangData {

        private int langInteger;
        private Libelle libelle;

        private ThesaurusLangData(int langInteger) {
            this.langInteger = langInteger;
        }

        public Libelle getLibelle() {
            return libelle;
        }

        public int getLangInteger() {
            return langInteger;
        }

        private void setLibelle(Libelle libelle) {
            this.libelle = libelle;
        }

    }


    public static class Builder {

        private ThesaurusURI thesaurusURI;
        private List<Libelle> libelleList = new ArrayList<Libelle>();
        private List<MotcleData.Builder> motcleDataBuilderList = new ArrayList<MotcleData.Builder>();

        public Builder(ThesaurusURI thesaurusURI) {
            this.thesaurusURI = thesaurusURI;
        }

        public ThesaurusURI getThesaurusURI() {
            return thesaurusURI;
        }

        public int getMotcleDataBuilderCount() {
            return motcleDataBuilderList.size();
        }

        public MotcleData.Builder getMotcleDataBuilder(int i) {
            return motcleDataBuilderList.get(i);
        }

        public void addLibelle(Libelle libelle) {
            if (libelle == null) {
                throw new IllegalArgumentException("libelle is null");
            }
            libelleList.add(libelle);
        }

        public void addMotcleDataBuilder(MotcleData.Builder motcleDataBuilder) {
            if (motcleDataBuilder == null) {
                throw new IllegalArgumentException("motcleInfoBuffer is null");
            }
            motcleDataBuilderList.add(motcleDataBuilder);
        }

        public ThesaurusData toThesaurusData(int codethesaurus, int codebase, int[] codesMotscles) {
            LinkedIntHashMap linkedHashMap = new LinkedIntHashMap();
            for (int i = 0; i < libelleList.size(); i++) {
                Libelle libelle = libelleList.get(i);
                int langInteger = libelle.getLangInteger();
                ThesaurusData.ThesaurusLangData langInfo = (ThesaurusData.ThesaurusLangData) linkedHashMap.get(langInteger);
                if (langInfo == null) {
                    langInfo = new ThesaurusData.ThesaurusLangData(langInteger);
                    linkedHashMap.put(langInteger, langInfo);
                }
                langInfo.setLibelle(libelle);
            }
            int size = motcleDataBuilderList.size();
            return new ThesaurusData(thesaurusURI, codethesaurus, codebase, codesMotscles, linkedHashMap);
        }

    }

}
