/* ScrutariLib_SearchEngine - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine;

import net.mapeadores.scrutari.searchengine.pertinence.FichePertinence;
import net.mapeadores.scrutari.searchengine.pertinence.MotclePertinence;
import net.mapeadores.scrutari.searchengine.seeker.SeekHandler;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.SubstringPosition;


/**
 *
 * @author Vincent Calame
 */
public class EngineSeekHandler implements SeekHandler {

    private int searchTokenCount = 0;
    private LinkedIntHashMap motcleCodeMap = new LinkedIntHashMap();
    private LinkedIntHashMap ficheCodeMap = new LinkedIntHashMap();

    public EngineSeekHandler(int searchTokenCount) {
        this.searchTokenCount = searchTokenCount;
    }

    public void addFichePertinence(int codefiche, int codecorpus, int tokenIndex, int fieldNumber, SubstringPosition sourcePosition) {
        FichePertinence pertinence = (FichePertinence) ficheCodeMap.get(codefiche);
        if (pertinence == null) {
            pertinence = new FichePertinence(codefiche, codecorpus, searchTokenCount);
            ficheCodeMap.put(codefiche, pertinence);
        }
        pertinence.addPertinence(tokenIndex, fieldNumber, sourcePosition);
    }

    public void addMotclePertinence(int codemotcle, int codethesaurus, int tokenIndex, int langInteger, SubstringPosition sourcePosition, float coverRate) {
        MotclePertinence pertinence = (MotclePertinence) motcleCodeMap.get(codemotcle);
        if (pertinence == null) {
            pertinence = new MotclePertinence(codemotcle, codethesaurus, searchTokenCount);
            motcleCodeMap.put(codemotcle, pertinence);
        }
        pertinence.addPertinence(tokenIndex, langInteger, sourcePosition, coverRate);
    }

    LinkedIntHashMap getMotcleCodeMap() {
        return motcleCodeMap;
    }

    LinkedIntHashMap getFicheCodeMap() {
        return ficheCodeMap;
    }

    void checkMotclePertinence(int uiLangInteger) {
        if ((uiLangInteger == 0) || (uiLangInteger == LangInteger.UNDETERMINED)) {
            return;
        }
        for (int i = 0; i < motcleCodeMap.size(); i++) {
            MotclePertinence motclePertinence = (MotclePertinence) motcleCodeMap.getValueAt(i);
            motclePertinence.finalizePertinence(uiLangInteger);
        }
    }

}
