/* ScrutariLib_SearchEngine - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine;

import net.mapeadores.scrutari.db.CodeFilter;
import net.mapeadores.scrutari.db.codefilter.CodeFilterUtils;
import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.db.CodeListUtils;
import net.mapeadores.scrutari.db.ElementCodeList;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.searchengine.util.SearchOptionsUtils;
import net.mapeadores.scrutari.supermotcle.Supermotcle;
import net.mapeadores.util.text.TextConstants;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;
import net.mapeadores.scrutari.db.DataAccess;


/**
 *
 *
 * @author  Vincent Calame
 */
public class SearchOptionsBuilder {

    private ScrutariDB scrutariDB;
    private int[] langIntegerArray = null;
    private int defaultSearchType = TextConstants.STARTSWITH;
    private int langUi;
    private CodeList codecorpusList;
    private CodeList codethesaurusList;
    private CodeFilter codeficheFilter;
    private CodeFilter codemotcleFilter;

    public SearchOptionsBuilder(ScrutariDB scrutariDB) {
        this.scrutariDB = scrutariDB;
    }

    public int getLangUi() {
        return langUi;
    }

    public void setLangUi(int langUi) {
        this.langUi = langUi;
    }

    public int[] getLangIntegerArray() {
        return langIntegerArray;
    }

    /**
     * Nul ou de longueur non nulle.
     *
     */
    public void setLangIntegerArray(int[] langIntegerArray) {
        if ((langIntegerArray != null) && (langIntegerArray.length == 0)) {
            this.langIntegerArray = null;
        } else {
            this.langIntegerArray = langIntegerArray;
        }
    }

    public void setCodecorpusList(CodeList codecorpusList) {
        this.codecorpusList = codecorpusList;
    }

    public void setCodethesaurusList(CodeList codethesaurusList) {
        this.codethesaurusList = codethesaurusList;
    }

    public int getDefaultSearchType() {
        return defaultSearchType;
    }

    public void setDefaultSearchType(int defaultSearchType) {
        this.defaultSearchType = defaultSearchType;
    }

    public SearchOptions toSearchOptions() {
        InternalSearchOptions ise = new InternalSearchOptions();
        if ((codecorpusList != null) && (codethesaurusList == null)) {
            DataAccess dataAccess = scrutariDB.openDataAccess();
            codethesaurusList = SearchOptionsUtils.getSelectionThesaurusCodeList(scrutariDB.getCountStats(), dataAccess, codecorpusList);
            dataAccess.close();
        }
        ise.langIntegerArray = langIntegerArray;
        ise.codecorpusList = codecorpusList;
        ise.codethesaurusList = codethesaurusList;
        ise.codeficheFilter = codeficheFilter;
        ise.codemotcleFilter = codemotcleFilter;
        ise.defaultSearchType = defaultSearchType;
        ise.langUi = langUi;
        return ise;
    }

    public void setCodeficheFilter(CodeFilter codeficheFilter) {
        this.codeficheFilter = codeficheFilter;
    }

    public void setCodemotcleFilter(CodeFilter codemotcleFilter) {
        this.codemotcleFilter = codemotcleFilter;
    }


    private static class InternalSearchOptions implements SearchOptions {

        private int[] langIntegerArray = null;
        private CodeList codecorpusList;
        private CodeList codethesaurusList;
        private CodeFilter codeficheFilter;
        private CodeFilter codemotcleFilter;
        private int defaultSearchType = TextConstants.STARTSWITH;
        private int langUi;

        public CodeList getCodecorpusList() {
            return codecorpusList;
        }

        public int getDefaultSearchType() {
            return defaultSearchType;
        }

        public CodeList getCodethesaurusList() {
            return codethesaurusList;
        }

        public int getLangUi() {
            return langUi;
        }

        public int[] getLangIntegerArray() {
            return langIntegerArray;
        }

        public CodeFilter getCodeficheFilter() {
            return codeficheFilter;
        }

        public CodeFilter getCodemotcleFilter() {
            return codemotcleFilter;
        }

    }

    public void setCodemotcleFilterFromSupermotcle(Supermotcle supermotcle) {
        Set<Integer> currentThsSet = null;
        if (codethesaurusList != null) {
            currentThsSet = new LinkedHashSet<Integer>();
            int thesaurusCount = codethesaurusList.getCodeCount();
            for (int i = 0; i < thesaurusCount; i++) {
                currentThsSet.add(codethesaurusList.getCode(i));
            }
        }
        new LinkedHashSet<Integer>();
        Set<Integer> thesaurusSet = new LinkedHashSet<Integer>();
        Set<Integer> motcleSet = new HashSet<Integer>();
        ElementCodeList codemotcleList = supermotcle.getCodemotcleList();
        int motcleCount = codemotcleList.getCodeCount();
        for (int i = 0; i < motcleCount; i++) {
            int codemotcle = codemotcleList.getCode(i);
            int codethesaurus = codemotcleList.getSubsetCode(i);
            if ((currentThsSet != null) && (!currentThsSet.contains(codethesaurus))) {
                continue;
            }
            thesaurusSet.add(codethesaurus);
            motcleSet.add(codemotcle);
        }
        codethesaurusList = CodeListUtils.toCodeList(thesaurusSet);
        codemotcleFilter = CodeFilterUtils.toCodeFilter(motcleSet);
    }

}
