/* ScrutariLib_SearchEngine - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine;

import net.mapeadores.scrutari.searchengine.impl.MutableFicheSearchTokenPositionList;
import net.mapeadores.scrutari.searchengine.impl.MutableMotcleSearchTokenPositionList;
import net.mapeadores.scrutari.searchengine.pertinence.FichePertinence;
import net.mapeadores.scrutari.searchengine.pertinence.IndexationPertinence;
import net.mapeadores.scrutari.searchengine.pertinence.MotclePertinence;
import net.mapeadores.scrutari.searchengine.util.SearchTokenPositionArrayBuilder;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.text.SubstringPosition;


/**
 *
 * @author Vincent Calame
 */
public class SearchUtils {

    public final static SupermotcleSearchResult EMPTY_SUPERMOTCLE_SEARCHRESULT = new EmptySupermotcleSearchResult();

    public static int[] getAcceptedCodeficheArray(IndexationPertinence indexationPertinence) {
        int acceptedFicheCount = indexationPertinence.getAcceptedFicheCount();
        int[] acceptedCodeficheArray = new int[acceptedFicheCount];
        for (int i = 0; i < acceptedFicheCount; i++) {
            acceptedCodeficheArray[i] = indexationPertinence.getAcceptedFiche(i).getCodeFiche();
        }
        return acceptedCodeficheArray;
    }

    public static int[] getMotcleCodeArray(FichePertinence fichePertinence) {
        if (fichePertinence.getMotcleCompletudeMask() < 1) {
            return null;
        }
        MotclePertinence[] pertinenceArray = fichePertinence.getMotclePertinenceByTokenArray();
        int length = pertinenceArray.length;
        int[] codesmotscles = new int[length];
        if (length == 1) {
            codesmotscles[0] = pertinenceArray[0].getCodeMotcle();
        } else {
            int p = 0;
            for (int i = 0; i < length; i++) {
                if (pertinenceArray[i] != null) {
                    int codemotcle = pertinenceArray[i].getCodeMotcle();
                    boolean here = false;
                    for (int j = 0; j < p; j++) {
                        if (codesmotscles[j] == codemotcle) {
                            here = true;
                            break;
                        }
                    }
                    if (!here) {
                        codesmotscles[p] = codemotcle;
                        p++;
                    }
                }
            }
            if (p < length) {
                int[] nv = new int[p];
                System.arraycopy(codesmotscles, 0, nv, 0, p);
                codesmotscles = nv;
            }
        }
        return codesmotscles;
    }

    public static MotcleSearchTokenPositionList[] getMotcleSearchTokenPositionListArray(MotclePertinence motclePertinence) {
        LinkedIntHashMap langMap = new LinkedIntHashMap();
        int concernedLangCount = motclePertinence.getConcernedLangCount();
        for (int i = 0; i < concernedLangCount; i++) {
            int concernedLangInteger = motclePertinence.getConcernedLangInteger(i);
            SubstringPosition[] array = motclePertinence.getSubstringPositionByTokenArray(i);
            int tokenCount = array.length;
            SearchTokenPositionArrayBuilder builder = (SearchTokenPositionArrayBuilder) langMap.get(concernedLangInteger);
            if (builder == null) {
                builder = new SearchTokenPositionArrayBuilder(tokenCount);
                langMap.put(concernedLangInteger, builder);
            }
            for (int tokenIndex = 0; tokenIndex < tokenCount; tokenIndex++) {
                SubstringPosition substringPosition = array[tokenIndex];
                if (substringPosition != null) {
                    builder.add(tokenIndex, substringPosition);
                }
            }
        }
        int size = langMap.size();
        MotcleSearchTokenPositionList[] beanArray = new MotcleSearchTokenPositionList[size];
        for (int i = 0; i < size; i++) {
            int langInteger = langMap.getKeyAt(i);
            SearchTokenPositionArrayBuilder builder = (SearchTokenPositionArrayBuilder) langMap.getValueAt(i);
            MutableMotcleSearchTokenPositionList mstpl = new MutableMotcleSearchTokenPositionList();
            mstpl.setLangInteger(langInteger);
            mstpl.setAllTokens(builder.isAllTokens());
            mstpl.setSearchTokenPositionArray(builder.getSearchTokenPositionArray());
            beanArray[i] = mstpl;
        }
        return beanArray;
    }

    public static FicheSearchTokenPositionList[] getFicheSearchTokenPositionListArray(FichePertinence fichePertinence) {
        int[] fieldNumbersByToken = fichePertinence.getFieldNumberByTokenArray();
        SubstringPosition[] substringPositionByToken = fichePertinence.getSubstringPositionByTokenArray();
        int tokenCount = fieldNumbersByToken.length;

        LinkedIntHashMap intHashMap = new LinkedIntHashMap();

        for (int tokenIndex = 0; tokenIndex < tokenCount; tokenIndex++) {
            SubstringPosition substringPosition = substringPositionByToken[tokenIndex];
            if (substringPosition == null) {
                continue;
            }
            int fieldNumber = fieldNumbersByToken[tokenIndex];
            SearchTokenPositionArrayBuilder builder = (SearchTokenPositionArrayBuilder) intHashMap.get(fieldNumber);
            if (builder == null) {
                builder = new SearchTokenPositionArrayBuilder(tokenCount);
                intHashMap.put(fieldNumber, builder);
            }
            builder.add(tokenIndex, substringPosition);
        }
        int size = intHashMap.size();
        FicheSearchTokenPositionList[] beanArray = new FicheSearchTokenPositionList[size];
        for (int i = 0; i < size; i++) {
            int fieldNumber = intHashMap.getKeyAt(i);
            SearchTokenPositionArrayBuilder builder = (SearchTokenPositionArrayBuilder) intHashMap.getValueAt(i);
            MutableFicheSearchTokenPositionList fstpl = new MutableFicheSearchTokenPositionList();
            fstpl.setFieldNumber(fieldNumber);
            fstpl.setAllTokens(builder.isAllTokens());
            fstpl.setSearchTokenPositionArray(builder.getSearchTokenPositionArray());
            beanArray[i] = fstpl;
        }
        return beanArray;
    }


    private static class EmptySupermotcleSearchResult implements SupermotcleSearchResult {

        public int getSupermotcleSearchResultInfoCount() {
            return 0;
        }

        public SupermotcleSearchResultInfo getSupermotcleSearchResultInfo(int i) {
            throw new IndexOutOfBoundsException();
        }

    }

}
