/* ScrutariLib_SearchEngine - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine;

import net.mapeadores.scrutari.searchengine.supermotcle.SupermotcleSeeker;
import net.mapeadores.scrutari.supermotcle.SupermotcleManager;
import net.mapeadores.scrutari.supermotcle.SupermotcleMap;
import net.mapeadores.util.localisation.LocalisationLexieFilters;
import net.mapeadores.util.text.SearchSequence;


/**
 *
 * @author Vincent Calame
 */
public class SupermotcleSearchEngine {

    private SupermotcleMap supermotcleMap;
    private SearchSequence filteredSearchSequence;

    private SupermotcleSearchEngine(SearchEngineParameters searchEngineParameters) {
        SearchOptions searchOptions = searchEngineParameters.getSearchOptions();
        int langInteger = searchOptions.getLangIntegerArray()[0];
        SupermotcleManager supermotcleManager = searchEngineParameters.getSupermotcleManager();
        supermotcleMap = supermotcleManager.getSupermotcleMap(langInteger);
        SearchSequence searchSequence = searchEngineParameters.getSearchSequence();
        this.filteredSearchSequence = searchSequence.filter(LocalisationLexieFilters.getLexieFilter(langInteger), searchOptions.getDefaultSearchType());
    }

    private SupermotcleSearchResult getResult() {
        if (supermotcleMap == null) {
            return SearchUtils.EMPTY_SUPERMOTCLE_SEARCHRESULT;
        }
        return SupermotcleSeeker.search(supermotcleMap, filteredSearchSequence);
    }

    public static SupermotcleSearchResult search(SearchEngineParameters searchEngineParameters) {
        SupermotcleSearchEngine searchEngine = new SupermotcleSearchEngine(searchEngineParameters);
        return searchEngine.getResult();
    }

}
