/* ScrutariLib_SearchEngine - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine.inner;

import net.mapeadores.scrutari.searchengine.FicheSearchResult;
import net.mapeadores.scrutari.searchengine.FicheSearchResultGroup;
import net.mapeadores.scrutari.searchengine.FicheSearchResultInfo;
import net.mapeadores.scrutari.searchengine.FicheSortEngine;
import net.mapeadores.scrutari.searchengine.FicheSortGroup;
import net.mapeadores.scrutari.searchengine.MotcleSearchResultInfo;
import net.mapeadores.scrutari.searchengine.SearchOptions;
import net.mapeadores.scrutari.searchengine.SearchUtils;
import net.mapeadores.scrutari.searchengine.impl.MutableFicheSearchResult;
import net.mapeadores.scrutari.searchengine.impl.MutableFicheSearchResultGroup;
import net.mapeadores.scrutari.searchengine.impl.MutableFicheSearchResultInfo;
import net.mapeadores.scrutari.searchengine.impl.MutableMotcleSearchResultInfo;
import net.mapeadores.scrutari.searchengine.pertinence.FichePertinence;
import net.mapeadores.scrutari.searchengine.pertinence.IndexationPertinence;
import net.mapeadores.scrutari.searchengine.pertinence.MotclePertinence;
import net.mapeadores.util.text.SearchSequence;


/**
 * Classe de construction d'un résultat d'une recherche.
 *
 * @author  Vincent Calame
 */
public class SearchResultBuilder {

    private String identifiant;
    private SearchSequence sequence;
    private SearchOptions options;
    private int ficheCount = 0;
    private int ficheMaximum = 0;
    private int motcleCount = 0;
    private int completeMotcleCount = 0;
    private FicheSearchResultGroup[] ficheSearchResultGroupArray;
    private MotcleSearchResultInfo[] motcleResults;
    private int pasmotcle = 0;

    public SearchResultBuilder(String identifiant, SearchSequence sequence, SearchOptions options) {
        this.identifiant = identifiant;
        this.sequence = sequence;
        this.options = options;
    }

    public FicheSearchResult toFicheSearchResult() {
        MutableFicheSearchResult ficheSearchResult = new MutableFicheSearchResult();
        ficheSearchResult.setIdentifiant(identifiant);
        ficheSearchResult.setSearchSequence(sequence);
        ficheSearchResult.setLangUi(options.getLangUi());
        ficheSearchResult.setLangIntegerArray(options.getLangIntegerArray());
        ficheSearchResult.setFicheCount(ficheCount);
        ficheSearchResult.setFicheMaximum(ficheMaximum);
        ficheSearchResult.setMotcleCount(motcleCount);
        ficheSearchResult.setCompleteMotcleCount(completeMotcleCount);
        ficheSearchResult.setFicheSearchResultGroupArray(ficheSearchResultGroupArray);
        ficheSearchResult.setMotcleSearchResultInfoArray(motcleResults);
        return ficheSearchResult;
    }

    public void initMotcleCount(int motcleCount) {
        this.motcleCount = motcleCount;
        motcleResults = new MotcleSearchResultInfo[motcleCount];
        pasmotcle = 0;
    }

    public void setCompleteMotcleCount(int completeMotcleCount) {
        this.completeMotcleCount = completeMotcleCount;
    }

    public void setFicheMaximum(int ficheMaximum) {
        this.ficheMaximum = ficheMaximum;
    }

    public void initFicheResult(FicheSortEngine ficheSortEngine) {
        this.ficheCount = ficheSortEngine.getFicheCount();
        FicheSortGroup[] ficheSortGroupArray = ficheSortEngine.getFicheSortGroupArray();
        if (ficheSortGroupArray != null) {
            int length = ficheSortGroupArray.length;
            ficheSearchResultGroupArray = new FicheSearchResultGroup[length];
            for (int i = 0; i < length; i++) {
                FicheSortGroup ficheSortGroup = ficheSortGroupArray[i];
                FichePertinence[] array = ficheSortGroup.toFichePertinenceArray();
                int ln2 = array.length;
                FicheSearchResultInfo[] ficheSearchResultInfoArray = new FicheSearchResultInfo[ln2];
                for (int j = 0; j < ln2; j++) {
                    ficheSearchResultInfoArray[j] = toFicheSearchResultInfo(array[j]);
                }
                MutableFicheSearchResultGroup ficheSearchResultGroup = new MutableFicheSearchResultGroup();
                ficheSearchResultGroup.setCorpusClasseRank(ficheSortGroup.getCorpusClasseRank());
                ficheSearchResultGroup.setFicheSearchResultInfoArray(ficheSearchResultInfoArray);
                ficheSearchResultGroupArray[i] = ficheSearchResultGroup;
            }
        }
    }

    /*
     * Ajoute une nouvelle instance de MotcleResult à partir de l'objet MotclePertinence en
     * argument.
     */
    public void addMotcleResult(IndexationPertinence indexationPertinence) {
        motcleResults[pasmotcle] = toMotcleSearchResultInfo(indexationPertinence);
        pasmotcle++;
    }

    private MotcleSearchResultInfo toMotcleSearchResultInfo(IndexationPertinence indexationPertinence) {
        MutableMotcleSearchResultInfo motcleSearchResultInfo = new MutableMotcleSearchResultInfo();
        MotclePertinence motclePertinence = indexationPertinence.getMotclePertinence();
        motcleSearchResultInfo.setCode(motclePertinence.getCodeMotcle());
        motcleSearchResultInfo.setMotcleSearchTokenPositionListArray(SearchUtils.getMotcleSearchTokenPositionListArray(motclePertinence));
        motcleSearchResultInfo.setAcceptedFicheCodeArray(SearchUtils.getAcceptedCodeficheArray(indexationPertinence));
        motcleSearchResultInfo.setFicheMaximum(indexationPertinence.getFicheMaximum());
        return motcleSearchResultInfo;
    }

    private FicheSearchResultInfo toFicheSearchResultInfo(FichePertinence fichePertinence) {
        MutableFicheSearchResultInfo ficheSearchResultInfo = new MutableFicheSearchResultInfo();
        ficheSearchResultInfo.setCode(fichePertinence.getCodeFiche());
        ficheSearchResultInfo.setMotcleCodeArray(SearchUtils.getMotcleCodeArray(fichePertinence));
        ficheSearchResultInfo.setFicheSearchTokenPositionListArray(SearchUtils.getFicheSearchTokenPositionListArray(fichePertinence));
        ficheSearchResultInfo.setPertinence(fichePertinence.getPertinenceRate());
        return ficheSearchResultInfo;
    }

}
