/* ScrutariLib_SearchEngine - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine.io;

import java.io.IOException;
import net.mapeadores.scrutari.searchengine.FicheSearchResult;
import net.mapeadores.scrutari.searchengine.FicheSearchResultGroup;
import net.mapeadores.scrutari.searchengine.FicheSearchResultInfo;
import net.mapeadores.scrutari.searchengine.FicheSearchTokenPositionList;
import net.mapeadores.scrutari.searchengine.MotcleSearchResultInfo;
import net.mapeadores.scrutari.searchengine.MotcleSearchTokenPositionList;
import net.mapeadores.scrutari.searchengine.SearchTokenPositionList;
import net.mapeadores.scrutari.searchengine.impl.MutableFicheSearchResult;
import net.mapeadores.scrutari.searchengine.impl.MutableFicheSearchResultGroup;
import net.mapeadores.scrutari.searchengine.impl.MutableFicheSearchResultInfo;
import net.mapeadores.scrutari.searchengine.impl.MutableFicheSearchTokenPositionList;
import net.mapeadores.scrutari.searchengine.impl.MutableMotcleSearchResultInfo;
import net.mapeadores.scrutari.searchengine.impl.MutableMotcleSearchTokenPositionList;
import net.mapeadores.scrutari.searchengine.impl.SearchTokenPosition;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.SearchSequence;
import net.mapeadores.util.text.SubstringPosition;


/**
 *
 * @author Vincent Calame
 */
public class FicheSearchResultStorage {

    private FicheSearchResultStorage() {
    }

    public static void write(PrimitivesWriter primitivesWriter, FicheSearchResult ficheSearchResult) throws IOException {
        primitivesWriter.writeString(ficheSearchResult.getIdentifiant());
        writeSearchSequence(primitivesWriter, ficheSearchResult.getSearchSequence());
        primitivesWriter.writeInt(ficheSearchResult.getLangUi());
        int[] langIntegerArray = ficheSearchResult.getLangIntegerArray();
        if (langIntegerArray == null) {
            primitivesWriter.writeInt(0);
        } else {
            int length = langIntegerArray.length;
            primitivesWriter.writeInt(length);
            for (int i = 0; i < length; i++) {
                primitivesWriter.writeInt(langIntegerArray[i]);
            }
        }
        primitivesWriter.writeInt(ficheSearchResult.getFicheCount());
        primitivesWriter.writeInt(ficheSearchResult.getFicheMaximum());
        int motcleCount = ficheSearchResult.getMotcleCount();
        primitivesWriter.writeInt(motcleCount);
        primitivesWriter.writeInt(ficheSearchResult.getCompleteMotcleCount());
        int ficheSearchResultGroupCount = ficheSearchResult.getFicheSearchResultGroupCount();
        primitivesWriter.writeInt(ficheSearchResultGroupCount);
        for (int i = 0; i < ficheSearchResultGroupCount; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = ficheSearchResult.getFicheSearchResultGroup(i);
            primitivesWriter.writeInt(ficheSearchResultGroup.getCorpusClasseRank());
            int count = ficheSearchResultGroup.getFicheSearchResultCount();
            primitivesWriter.writeInt(count);
            for (int j = 0; j < count; j++) {
                writeFicheSearchResultInfo(primitivesWriter, ficheSearchResultGroup.getFicheSearchResultInfo(j));
            }
        }
        for (int i = 0; i < motcleCount; i++) {
            writeMotcleSearchResultInfo(primitivesWriter, ficheSearchResult.getMotcleResult(i));
        }
    }

    public static FicheSearchResult read(PrimitivesReader primitivesReader) throws IOException {
        String identifiant = primitivesReader.readString();
        SearchSequence searchSequence = readSearchSequence(primitivesReader);
        int langUi = primitivesReader.readInt();
        int langIntegerArrayLength = primitivesReader.readInt();
        int[] langIntegerArray = null;
        if (langIntegerArrayLength > 0) {
            langIntegerArray = new int[langIntegerArrayLength];
            for (int i = 0; i < langIntegerArrayLength; i++) {
                langIntegerArray[i] = primitivesReader.readInt();
            }
        }
        int ficheCount = primitivesReader.readInt();
        int ficheMaximum = primitivesReader.readInt();
        int motcleCount = primitivesReader.readInt();
        int completeMotcleCount = primitivesReader.readInt();
        int ficheSearchResultGroupCount = primitivesReader.readInt();
        FicheSearchResultGroup[] ficheSearchResultGroupArray = null;
        if (ficheCount > 0) {
            ficheSearchResultGroupArray = new FicheSearchResultGroup[ficheSearchResultGroupCount];
            for (int i = 0; i < ficheSearchResultGroupCount; i++) {
                int corpusClasseRank = primitivesReader.readInt();
                int count = primitivesReader.readInt();
                FicheSearchResultInfo[] ficheSearchResultInfoArray = new FicheSearchResultInfo[count];
                for (int j = 0; j < count; j++) {
                    ficheSearchResultInfoArray[j] = readFicheSearchResultInfo(primitivesReader);
                }
                MutableFicheSearchResultGroup ficheSearchResultGroup = new MutableFicheSearchResultGroup();
                ficheSearchResultGroup.setCorpusClasseRank(corpusClasseRank);
                ficheSearchResultGroup.setFicheSearchResultInfoArray(ficheSearchResultInfoArray);
                ficheSearchResultGroupArray[i] = ficheSearchResultGroup;
            }
        }
        MotcleSearchResultInfo[] motcleSearchResultInfoArray = null;
        if (motcleCount > 0) {
            motcleSearchResultInfoArray = new MotcleSearchResultInfo[motcleCount];
            for (int i = 0; i < motcleCount; i++) {
                motcleSearchResultInfoArray[i] = readMotcleSearchResultInfo(primitivesReader);
            }
        }
        MutableFicheSearchResult ficheSearchResult = new MutableFicheSearchResult();
        ficheSearchResult.setIdentifiant(identifiant);
        ficheSearchResult.setSearchSequence(searchSequence);
        ficheSearchResult.setLangIntegerArray(langIntegerArray);
        ficheSearchResult.setLangUi(langUi);
        ficheSearchResult.setFicheCount(ficheCount);
        ficheSearchResult.setFicheMaximum(ficheMaximum);
        ficheSearchResult.setMotcleCount(motcleCount);
        ficheSearchResult.setCompleteMotcleCount(completeMotcleCount);
        ficheSearchResult.setFicheSearchResultGroupArray(ficheSearchResultGroupArray);
        ficheSearchResult.setMotcleSearchResultInfoArray(motcleSearchResultInfoArray);
        return ficheSearchResult;
    }

    private static void writeFicheSearchResultInfo(PrimitivesWriter primitivesWriter, FicheSearchResultInfo ficheSearchResultInfo) throws IOException {
        primitivesWriter.writeInt(ficheSearchResultInfo.getCode());
        int motcleCount = ficheSearchResultInfo.getMotcleCount();
        primitivesWriter.writeInt(motcleCount);
        for (int i = 0; i < motcleCount; i++) {
            primitivesWriter.writeInt(ficheSearchResultInfo.getMotcleCode(i));
        }
        int ficheSearchTokenPositionListCount = ficheSearchResultInfo.getFicheSearchTokenPositionListCount();
        primitivesWriter.writeInt(ficheSearchTokenPositionListCount);
        for (int i = 0; i < ficheSearchTokenPositionListCount; i++) {
            FicheSearchTokenPositionList ficheSearchTokenPositionList = ficheSearchResultInfo.getFicheSearchTokenPositionList(i);
            primitivesWriter.writeInt(ficheSearchTokenPositionList.getFieldNumber());
            writeSearchTokenPositionList(primitivesWriter, ficheSearchTokenPositionList);
        }
        primitivesWriter.writeFloat(ficheSearchResultInfo.getPertinence());
    }

    private static void writeMotcleSearchResultInfo(PrimitivesWriter primitivesWriter, MotcleSearchResultInfo motcleSearchResultInfo) throws IOException {
        primitivesWriter.writeInt(motcleSearchResultInfo.getCode());
        int acceptedFicheCount = motcleSearchResultInfo.getAcceptedFicheCount();
        primitivesWriter.writeInt(acceptedFicheCount);
        for (int i = 0; i < acceptedFicheCount; i++) {
            primitivesWriter.writeInt(motcleSearchResultInfo.getAcceptedFicheCode(i));
        }
        primitivesWriter.writeInt(motcleSearchResultInfo.getFicheMaximum());
        int motcleSearchTokenPositionListCount = motcleSearchResultInfo.getMotcleSearchTokenPositionListCount();
        primitivesWriter.writeInt(motcleSearchTokenPositionListCount);
        for (int i = 0; i < motcleSearchTokenPositionListCount; i++) {
            MotcleSearchTokenPositionList motcleSearchTokenPositionList = motcleSearchResultInfo.getMotcleSearchTokenPositionList(i);
            primitivesWriter.writeInt(motcleSearchTokenPositionList.getLangInteger());
            writeSearchTokenPositionList(primitivesWriter, motcleSearchTokenPositionList);
        }
    }

    private static void writeSearchTokenPositionList(PrimitivesWriter primitivesWriter, SearchTokenPositionList searchTokenPositionList) throws IOException {
        primitivesWriter.writeBoolean(searchTokenPositionList.containsAllSearchTokens());
        int size = searchTokenPositionList.size();
        primitivesWriter.writeInt(size);
        for (int j = 0; j < size; j++) {
            SubstringPosition substringPosition = searchTokenPositionList.getSearchTokenPosition(j);
            writeSubstringPosition(primitivesWriter, substringPosition);
            primitivesWriter.writeInt(searchTokenPositionList.getSearchTokenIndexForPosition(j));
        }
    }

    private static FicheSearchResultInfo readFicheSearchResultInfo(PrimitivesReader primitivesReader) throws IOException {
        int code = primitivesReader.readInt();
        int motcleCount = primitivesReader.readInt();
        int[] motcleCodeArray = null;
        if (motcleCount > 0) {
            motcleCodeArray = new int[motcleCount];
            for (int i = 0; i < motcleCount; i++) {
                motcleCodeArray[i] = primitivesReader.readInt();
            }
        }
        int ficheSearchTokenPositionListCount = primitivesReader.readInt();
        FicheSearchTokenPositionList[] ficheSearchTokenPositionListArray = null;
        if (ficheSearchTokenPositionListCount > 0) {
            ficheSearchTokenPositionListArray = new FicheSearchTokenPositionList[ficheSearchTokenPositionListCount];
            for (int i = 0; i < ficheSearchTokenPositionListCount; i++) {
                int fieldNumber = primitivesReader.readInt();
                boolean allSearchToken = primitivesReader.readBoolean();
                SearchTokenPosition[] searchTokenPositionArray = readSearchTokenPositionArray(primitivesReader);
                MutableFicheSearchTokenPositionList ficheSearchTokenPositionList = new MutableFicheSearchTokenPositionList();
                ficheSearchTokenPositionList.setFieldNumber(fieldNumber);
                ficheSearchTokenPositionList.setAllTokens(allSearchToken);
                ficheSearchTokenPositionList.setSearchTokenPositionArray(searchTokenPositionArray);
                ficheSearchTokenPositionListArray[i] = ficheSearchTokenPositionList;
            }
        }
        float pertinence = primitivesReader.readFloat();
        MutableFicheSearchResultInfo ficheSearchResultInfo = new MutableFicheSearchResultInfo();
        ficheSearchResultInfo.setCode(code);
        ficheSearchResultInfo.setMotcleCodeArray(motcleCodeArray);
        ficheSearchResultInfo.setFicheSearchTokenPositionListArray(ficheSearchTokenPositionListArray);
        ficheSearchResultInfo.setPertinence(pertinence);
        return ficheSearchResultInfo;
    }

    private static MotcleSearchResultInfo readMotcleSearchResultInfo(PrimitivesReader primitivesReader) throws IOException {
        int code = primitivesReader.readInt();
        int acceptedFicheCount = primitivesReader.readInt();
        int[] acceptedFicheCodeArray = null;
        if (acceptedFicheCount > 0) {
            acceptedFicheCodeArray = new int[acceptedFicheCount];
            for (int i = 0; i < acceptedFicheCount; i++) {
                acceptedFicheCodeArray[i] = primitivesReader.readInt();
            }
        }
        int ficheMaximum = primitivesReader.readInt();
        int motcleSearchTokenPositionListCount = primitivesReader.readInt();
        MotcleSearchTokenPositionList[] motcleSearchTokenPositionListArray = null;
        if (motcleSearchTokenPositionListCount > 0) {
            motcleSearchTokenPositionListArray = new MotcleSearchTokenPositionList[motcleSearchTokenPositionListCount];
            for (int i = 0; i < motcleSearchTokenPositionListCount; i++) {
                int langInteger = primitivesReader.readInt();
                boolean allSearchToken = primitivesReader.readBoolean();
                SearchTokenPosition[] searchTokenPositionArray = readSearchTokenPositionArray(primitivesReader);
                MutableMotcleSearchTokenPositionList motcleSearchTokenPositionList = new MutableMotcleSearchTokenPositionList();
                motcleSearchTokenPositionList.setLangInteger(langInteger);
                motcleSearchTokenPositionList.setAllTokens(allSearchToken);
                motcleSearchTokenPositionList.setSearchTokenPositionArray(searchTokenPositionArray);
                motcleSearchTokenPositionListArray[i] = motcleSearchTokenPositionList;
            }
        }
        MutableMotcleSearchResultInfo motcleSearchResultInfo = new MutableMotcleSearchResultInfo();
        motcleSearchResultInfo.setCode(code);
        motcleSearchResultInfo.setAcceptedFicheCodeArray(acceptedFicheCodeArray);
        motcleSearchResultInfo.setFicheMaximum(ficheMaximum);
        motcleSearchResultInfo.setMotcleSearchTokenPositionListArray(motcleSearchTokenPositionListArray);
        return motcleSearchResultInfo;
    }

    private static SearchTokenPosition[] readSearchTokenPositionArray(PrimitivesReader primitivesReader) throws IOException {
        int searchTokenPositionCount = primitivesReader.readInt();
        if (searchTokenPositionCount == 0) {
            return null;
        }
        SearchTokenPosition[] result = new SearchTokenPosition[searchTokenPositionCount];
        for (int i = 0; i < searchTokenPositionCount; i++) {
            SubstringPosition substringPosition = readSubstringPosition(primitivesReader);
            int searchTokenIndex = primitivesReader.readInt();
            SearchTokenPosition searchTokenPosition = new SearchTokenPosition();
            searchTokenPosition.setSubstringPosition(substringPosition);
            searchTokenPosition.setSearchTokenIndex(searchTokenIndex);
            result[i] = searchTokenPosition;
        }
        return result;
    }

    private static void writeSearchSequence(PrimitivesWriter primitivesWriter, SearchSequence searchSequence) throws IOException {
        primitivesWriter.writeString(searchSequence.getSource());
        int tokenCount = searchSequence.getTokenCount();
        primitivesWriter.writeInt(tokenCount);
        for (int i = 0; i < tokenCount; i++) {
            SearchSequence.Token token = searchSequence.getToken(i);
            if (token instanceof SearchSequence.SimpleToken) {
                SearchSequence.SimpleToken simpleToken = (SearchSequence.SimpleToken) token;
                primitivesWriter.writeChar('s');
                writeSimpleToken(primitivesWriter, simpleToken);
            } else {
                SearchSequence.MultiToken multiToken = (SearchSequence.MultiToken) token;
                primitivesWriter.writeChar('m');
                int subtokenCount = multiToken.getSubtokenCount();
                primitivesWriter.writeInt(subtokenCount);
                for (int j = 0; j < subtokenCount; j++) {
                    writeSimpleToken(primitivesWriter, multiToken.getSubtoken(j));
                }
            }
        }
        int[] nonPertinentIndexArray = searchSequence.getNonPertinentIndexArray();
        if (nonPertinentIndexArray == null) {
            primitivesWriter.writeInt(0);
        } else {
            int nonPertinentIndexLength = nonPertinentIndexArray.length;
            primitivesWriter.writeInt(nonPertinentIndexLength);
            for (int i = 0; i < nonPertinentIndexLength; i++) {
                primitivesWriter.writeInt(nonPertinentIndexArray[i]);
            }
        }
    }

    private static void writeSimpleToken(PrimitivesWriter primitivesWriter, SearchSequence.SimpleToken simpleToken) throws IOException {
        primitivesWriter.writeInt(simpleToken.getSearchType());
        primitivesWriter.writeString(simpleToken.getTokenString());
    }

    private static SearchSequence readSearchSequence(PrimitivesReader primitivesReader) throws IOException {
        String source = primitivesReader.readString();
        SearchSequence.Token[] tokens = readTokenArray(primitivesReader);
        int[] nonpertinentIndexArray = null;
        int nonpertinentIndexLength = primitivesReader.readInt();
        if (nonpertinentIndexLength > 0) {
            nonpertinentIndexArray = new int[nonpertinentIndexLength];
            for (int i = 0; i < nonpertinentIndexLength; i++) {
                nonpertinentIndexArray[i] = primitivesReader.readInt();
            }
        }
        return SearchSequence.newInstance(source, tokens, nonpertinentIndexArray);
    }

    private static SearchSequence.Token[] readTokenArray(PrimitivesReader primitivesReader) throws IOException {
        int count = primitivesReader.readInt();
        SearchSequence.Token[] tokenArray = new SearchSequence.Token[count];
        for (int i = 0; i < count; i++) {
            char type = primitivesReader.readChar();
            if (type == 's') {
                tokenArray[i] = readSimpleToken(primitivesReader, i);
            } else {
                int subtokenCount = primitivesReader.readInt();
                SearchSequence.SimpleToken[] subtokenArray = new SearchSequence.SimpleToken[subtokenCount];
                for (int j = 0; j < subtokenCount; j++) {
                    subtokenArray[j] = readSimpleToken(primitivesReader, j);
                }
                tokenArray[i] = new SearchSequence.MultiToken(i, subtokenArray);
            }
        }
        return tokenArray;
    }

    private static SearchSequence.SimpleToken readSimpleToken(PrimitivesReader primitivesReader, int tokenIndex) throws IOException {
        int searchType = primitivesReader.readInt();
        String tokenString = primitivesReader.readString();
        return new SearchSequence.SimpleToken(tokenIndex, tokenString, searchType);
    }

    private static void writeSubstringPosition(PrimitivesWriter primitivesWriter, SubstringPosition substringPosition) throws IOException {
        primitivesWriter.writeInt(substringPosition.getBeginIndex());
        primitivesWriter.writeInt(substringPosition.getLength());
    }

    private static SubstringPosition readSubstringPosition(PrimitivesReader primitivesReader) throws IOException {
        int beginIndex = primitivesReader.readInt();
        int length = primitivesReader.readInt();
        return new SubstringPosition(beginIndex, length);
    }

}
