/* ScrutariLib_SearchEngine - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine.pertinence;


/**
 *
 * @author Vincent Calame
 */
public class Comparaison {

    private Comparaison() {
    }

    /**
     * Compare le nombre de mots dans les mots-clés.
     */
    public static int searchTokenCountInMotscles(FichePertinence fp1, FichePertinence fp2) {
        int mcd1 = fp1.getSearchTokenCountInMotscles();
        int mcd2 = fp2.getSearchTokenCountInMotscles();
        if (mcd1 > mcd2) {
            return -1;
        } else if (mcd1 < mcd2) {
            return 1;
        } else {
            return 0;
        }
    }

    /**
     * Compare les pondérations maximales relevées pour les fiches
     */
    public static int ponderationMax(FichePertinence fp1, FichePertinence fp2) {
        float mrw1 = fp1.getPonderationMax();
        float mrw2 = fp2.getPonderationMax();
        if (mrw1 > mrw2) {
            return -1;
        } else if (mrw1 < mrw2) {
            return 1;
        } else {
            return 0;
        }
    }

    /**
     * Compare les dates
     */
    public static int date(FichePertinence fp1, FichePertinence fp2) {
        int date1 = fp1.getDateInteger();
        int date2 = fp2.getDateInteger();
        if (date1 > date2) {
            return -1;
        } else if (date1 < date2) {
            return 1;
        } else {
            return 0;
        }
    }

    /**
     * Compare l'année
     */
    public static int annee(FichePertinence fp1, FichePertinence fp2) {
        int date1 = fp1.getAnnee();
        int date2 = fp2.getAnnee();
        if (date1 > date2) {
            return -1;
        } else if (date1 < date2) {
            return 1;
        } else {
            return 0;
        }
    }

    /**
     * Compare la pertinence du mots-clé le mieux placé pour les deux fiches
     */
    public static int motclePertinenceMax(FichePertinence fp1, FichePertinence fp2) {
        MotclePertinence max1 = fp1.getMotclePertinenceMax();
        MotclePertinence max2 = fp2.getMotclePertinenceMax();
        if (max1 != null) {
            if (max2 == null) {
                return -1;
            } else {
                return max1.compareTo(max2);
            }
        } else if (max2 != null) {
            return 1;
        } else {
            return 0;
        }
    }

    public static int maxSearchTokenCountInMotcle(FichePertinence fp1, FichePertinence fp2) {
        MotclePertinence max1 = fp1.getMotclePertinenceMax();
        MotclePertinence max2 = fp2.getMotclePertinenceMax();
        if (max1 != null) {
            if (max2 == null) {
                return -1;
            } else {
                int stc1 = max1.getSearchTokenCount();
                int stc2 = max2.getSearchTokenCount();
                if (stc1 > stc2) {
                    return -1;
                } else if (stc1 < stc2) {
                    return 1;
                } else {
                    return 0;
                }
            }
        } else if (max2 != null) {
            return 1;
        } else {
            return 0;
        }
    }

    /**
     * Compare les codes de fiches. C'est la comparaison ultime qui n'a pas d'autre
     * sens que l'ordre d'entrée dans la base.
     */
    public static int code(FichePertinence fp1, FichePertinence fp2) {
        int cf1 = fp1.getCodeFiche();
        int cf2 = fp2.getCodeFiche();
        if (cf1 < cf2) {
            return -1;
        } else if (cf1 > cf2) {
            return 1;
        } else {
            return 0;
        }
    }

    /**
     * Compare la plus basse position parmi les mots de la recherche.
     * Comparaison de dernier recours lorsque les champs minimaux sont les mêmes
     */
    public static int minimalBeginIndex(FichePertinence fp1, FichePertinence fp2) {
        int mbi1 = fp1.getMinimalBeginIndex();
        int mbi2 = fp2.getMinimalBeginIndex();
        if (mbi1 < mbi2) {
            return -1;
        } else if (mbi1 > mbi2) {
            return 1;
        } else {
            return 0;
        }
    }

    /**
     * Compare la valeur des champs minimaux.
     */
    public static int minimalFieldNumber(FichePertinence fp1, FichePertinence fp2) {
        int mfm1 = fp1.getMinimalFieldNumber();
        int mfm2 = fp2.getMinimalFieldNumber();
        if (mfm1 < mfm2) {
            return -1;
        } else if (mfm1 > mfm2) {
            return 1;
        } else {
            return 0;
        }
    }

    /**
     * Compare le nombre de mots de la recherche dans le sous-titre
     */
    public static int searchTokenCountInSoustitre(FichePertinence fp1, FichePertinence fp2) {
        int stcd1 = fp1.getSearchTokenCountInSoustitre();
        int stcd2 = fp2.getSearchTokenCountInSoustitre();
        if (stcd1 > stcd2) {
            return -1;
        } else if (stcd1 < stcd2) {
            return 1;
        } else {
            return 0;
        }
    }

    /**
     * Compare le nombre de mots de la recherche dans le titre
     */
    public static int searchTokenCountInTitre(FichePertinence fp1, FichePertinence fp2) {
        int tcd1 = fp1.getSearchTokenCountInTitre();
        int tcd2 = fp2.getSearchTokenCountInTitre();
        if (tcd1 > tcd2) {
            return -1;
        } else if (tcd1 < tcd2) {
            return 1;
        } else {
            return 0;
        }
    }

}
