/* ScrutariLib_SearchEngine - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine.seeker;

import net.mapeadores.scrutari.db.LexieAccess;
import net.mapeadores.scrutari.lexie.FieldNumbers;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.localisation.LocalisationLexieFilters;
import net.mapeadores.util.text.SearchSequence;
import net.mapeadores.util.text.collation.map.CollatedKeyMap;
import net.mapeadores.util.text.collation.map.SearchResultUnitList;
import net.mapeadores.util.text.collation.map.ValueFilter;


/**
 *
 * @author Vincent Calame
 */
public class LexieSeeker {

    private LexieAccess lexieAccess;
    private LinkedIntHashMap ficheMap = new LinkedIntHashMap();
    private LinkedIntHashMap motcleMap = new LinkedIntHashMap();
    private SimpleTokenSeeker simpleTokenSeeker;
    private MultiTokenSeeker subtokenSeeker;
    private SeekOptions seekOptions;
    private SeekHandler seekHandler;
    private ValueFilter valueFilter = null;

    public LexieSeeker(SeekOptions seekOptions, SeekHandler seekHandler, LexieAccess lexieAccess) {
        this.lexieAccess = lexieAccess;
        this.seekOptions = seekOptions;
        this.seekHandler = seekHandler;
        simpleTokenSeeker = new SimpleTokenSeeker(this);
        subtokenSeeker = new MultiTokenSeeker(this);
    }

    public void seek(SearchSequence searchSequence) {
        int[] langIntegerArray = seekOptions.getLangIntegerArray();
        for (int k = 0; k < langIntegerArray.length; k++) {
            int langInteger = langIntegerArray[k];
            simpleTokenSeeker.setLangInteger(langInteger);
            subtokenSeeker.setLangInteger(langInteger);
            SearchSequence locSequence = searchSequence.filter(LocalisationLexieFilters.getLexieFilter(langInteger), seekOptions.getDefaultSearchType());
            ficheMap.clear();
            motcleMap.clear();
            int tokenCount = locSequence.getTokenCount();
            for (int i = 0; i < tokenCount; i++) {
                SearchSequence.Token token = locSequence.getToken(i);
                if (token.isMultiToken()) {
                    SearchSequence.MultiToken multiToken = (SearchSequence.MultiToken) token;
                    int count = multiToken.getSubtokenCount();
                    if (count > 1) {
                        subtokenSeeker.setToken(multiToken);
                        subtokenSeeker.seek();
                    } else if (count == 1) {
                        simpleTokenSeeker.setToken(multiToken.getSubtokenArray()[0]);
                        simpleTokenSeeker.seek();
                    }
                } else {
                    simpleTokenSeeker.setToken((SearchSequence.SimpleToken) token);
                    simpleTokenSeeker.seek();
                }
            }
            if (locSequence.hasNonPertinentTokens()) {
                int[] np = locSequence.getNonPertinentIndexArray();
                int npLength = np.length;
                int ficheMapSize = ficheMap.size();
                for (int i = 0; i < ficheMapSize; i++) {
                    int codefiche = ficheMap.getKeyAt(i);
                    int codecorpus = ((Integer) ficheMap.getValueAt(i)).intValue();
                    for (int j = 0; j < npLength; j++) {
                        seekHandler.addFichePertinence(codefiche, codecorpus, np[j], FieldNumbers.NOPERTINENT_FIELDNUMBER, null);
                    }
                }
                int motcleMapSize = motcleMap.size();
                for (int i = 0; i < motcleMapSize; i++) {
                    int codemotcle = motcleMap.getKeyAt(i);
                    int codethesaurus = ((Integer) motcleMap.getValueAt(i)).intValue();
                    for (int j = 0; j < npLength; j++) {
                        seekHandler.addMotclePertinence(codemotcle, codethesaurus, np[j], langInteger, null, 0.0f);
                    }
                }
            }
        }
    }

    LexieAccess getLexieAccess() {
        return lexieAccess;
    }

    SeekOptions getSeekOptions() {
        return seekOptions;
    }

    SeekHandler getSeekHandler() {
        return seekHandler;
    }

    void addFiche(int codefiche, int codecorpus) {
        ficheMap.put(codefiche, new Integer(codecorpus));
    }

    void addMotcle(int codemotcle, int codethesaurus) {
        motcleMap.put(codemotcle, new Integer(codethesaurus));
    }

    SearchResultUnitList search(String searchText, int langInteger, int type) {
        CollatedKeyMap collatedStringMap = lexieAccess.getCollatedLexieMapByLang(langInteger);
        if (collatedStringMap == null) {
            return null;
        }
        return collatedStringMap.search(searchText, type, valueFilter);
    }

}
