/* ScrutariLib_SearchEngine - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine.seeker;

import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.lexie.FicheLexification;
import net.mapeadores.scrutari.lexie.LexieOccurrences;
import net.mapeadores.scrutari.lexie.Lexification;
import net.mapeadores.scrutari.lexie.MotcleLexification;
import net.mapeadores.scrutari.db.CodeFilter;
import net.mapeadores.scrutari.db.LexieAccess;
import net.mapeadores.scrutari.lexie.LexieOccurrencesList;
import net.mapeadores.scrutari.searchengine.SearchFilter;
import net.mapeadores.util.text.SearchSequence;
import net.mapeadores.util.text.SubstringPosition;
import net.mapeadores.util.text.collation.map.SearchResultUnit;
import net.mapeadores.util.text.collation.map.SearchResultUnitList;


/**
 *
 * @author Vincent Calame
 */
public class MultiTokenSeeker {

    private LexieSeeker lexieSeeker;
    private LexieAccess lexieAccess;
    private SearchSequence.MultiToken multiToken;
    PotentialLexieSequence potentialLexieSequence;
    int langInteger;

    MultiTokenSeeker(LexieSeeker lexieSeeker) {
        this.lexieSeeker = lexieSeeker;
        this.lexieAccess = lexieSeeker.getLexieAccess();
    }

    public void setLangInteger(int langInteger) {
        this.langInteger = langInteger;
    }

    public void setToken(SearchSequence.MultiToken multiToken) {
        this.multiToken = multiToken;
    }

    void seek() {
        SearchSequence.SimpleToken[] subtokenArray = multiToken.getSubtokenArray();
        potentialLexieSequence = new PotentialLexieSequence(subtokenArray);
        if (!potentialLexieSequence.isInit()) {
            return;
        }
        int length = potentialLexieSequence.getPotentialStartCount();
        for (int i = 0; i < length; i++) {
            long lexieId = potentialLexieSequence.setCurrentPotentialStart(i);
            scanLexieOccurrences(lexieId);
        }
    }

    private void scanLexieOccurrences(long lexieId) {
        SeekOptions seekOptions = lexieSeeker.getSeekOptions();
        SearchFilter searchFilter = seekOptions.getSearchFilter();
        if (seekOptions.isWithFiches()) {
            CodeList codecorpusList = searchFilter.getCodecorpusList();
            CodeFilter codeficheFilter = searchFilter.getCodeficheFilter();
            int corpusCount = codecorpusList.getCodeCount();
            for (int j = 0; j < corpusCount; j++) {
                int codecorpus = codecorpusList.getCode(j);
                LexieOccurrencesList lexieOccurrencesList = lexieAccess.getLexieOccurrencesList(lexieId, codecorpus);
                int lexieCount = lexieOccurrencesList.getLexieOccurrencesCount();
                for (int k = 0; k < lexieCount; k++) {
                    LexieOccurrences lexieOccurrences = lexieOccurrencesList.getLexieOccurrences(k);
                    int codefiche = lexieOccurrences.getCode();
                    if (codeficheFilter.acceptCode(codefiche)) {
                        scanFicheLexification(codecorpus, lexieOccurrences);
                    }
                }
            }
        }
        if (seekOptions.isWithMotscles()) {
            CodeList codethesaurusList = searchFilter.getCodethesaurusList();
            CodeFilter codemotcleFilter = searchFilter.getCodemotcleFilter();
            int thesaurusCount = codethesaurusList.getCodeCount();
            for (int j = 0; j < thesaurusCount; j++) {
                int codethesaurus = codethesaurusList.getCode(j);
                LexieOccurrencesList lexieOccurrencesList = lexieAccess.getLexieOccurrencesList(lexieId, codethesaurus);
                int lexieCount = lexieOccurrencesList.getLexieOccurrencesCount();
                for (int k = 0; k < lexieCount; k++) {
                    LexieOccurrences lexieOccurrences = lexieOccurrencesList.getLexieOccurrences(k);
                    int codemotcle = lexieOccurrences.getCode();
                    if (codemotcleFilter.acceptCode(codemotcle)) {
                        scanMotcleLexification(codethesaurus, lexieOccurrences);
                    }
                }
            }
        }
    }

    private void scanFicheLexification(int codecorpus, LexieOccurrences lexieOccurrences) {
        SeekHandler seekHandler = lexieSeeker.getSeekHandler();
        int codefiche = lexieOccurrences.getCode();
        int count = lexieOccurrences.getFieldCount();
        FicheLexification ficheLexification = lexieAccess.getFicheLexification(codefiche);
        for (int i = 0; i < count; i++) {
            int fieldNumber = lexieOccurrences.getFieldNumber(i);
            Lexification lexification = ficheLexification.getLexification(fieldNumber);
            int[] array = lexieOccurrences.getLexieIndexInFieldLexification(i);
            int ln2 = array.length;
            for (int j = 0; j < ln2; j++) {
                int indexInLexification = array[j];
                SubstringPosition substringPosition = potentialLexieSequence.testLexification(lexification, indexInLexification);
                if (substringPosition != null) {
                    seekHandler.addFichePertinence(codefiche, codecorpus, multiToken.getTokenIndex(), fieldNumber, substringPosition);
                    lexieSeeker.addFiche(codefiche, codecorpus);
                }
            }
        }
    }

    private void scanMotcleLexification(int codethesaurus, LexieOccurrences lexieOccurrences) {
        SeekHandler seekHandler = lexieSeeker.getSeekHandler();
        int codemotcle = lexieOccurrences.getCode();
        MotcleLexification ficheLexification = lexieAccess.getMotcleLexification(codemotcle);
        Lexification lexification = ficheLexification.getLexification(langInteger);
        int[] array = lexieOccurrences.getLexieIndexInFieldLexification(0);
        int ln2 = array.length;
        for (int j = 0; j < ln2; j++) {
            int indexInLexification = array[j];
            SubstringPosition substringPosition = potentialLexieSequence.testLexification(lexification, indexInLexification);
            if (substringPosition != null) {
                float coverRate = ((float) substringPosition.getLength()) / (lexification.getWholeLength());
                seekHandler.addMotclePertinence(codemotcle, codethesaurus, multiToken.getTokenIndex(), langInteger, substringPosition, coverRate);
                lexieSeeker.addMotcle(codemotcle, codethesaurus);
            }
        }
    }


    private class PotentialLexieSequence {

        long[][] lexieIdArrays;
        SubstringPosition[] firstSearchTextePositionArray;
        SubstringPosition[] lastSearchTextePositionArray;
        private int currentPotentialStart = 0;

        PotentialLexieSequence(SearchSequence.SimpleToken[] subtokenArray) {
            int length = subtokenArray.length;
            lexieIdArrays = new long[length][];
            for (int i = 0; i < length; i++) {
                SearchSequence.SimpleToken token = subtokenArray[i];
                SearchResultUnitList list = lexieSeeker.search(token.toString(false), langInteger, token.getSearchType());
                if ((list == null) || (list.getSearchResultUnitCount() == 0)) {
                    lexieIdArrays = null;
                    return;
                }
                int size = list.getSearchResultUnitCount();
                long[] array = new long[size];
                SubstringPosition[] searchTextePositionArray = null;
                if (i == 0) {
                    firstSearchTextePositionArray = new SubstringPosition[size];
                    searchTextePositionArray = firstSearchTextePositionArray;
                } else if (i == (length - 1)) {
                    lastSearchTextePositionArray = new SubstringPosition[size];
                    searchTextePositionArray = lastSearchTextePositionArray;
                }
                for (int j = 0; j < size; j++) {
                    SearchResultUnit searchResultItem = list.getSearchResultUnit(j);
                    Long lexieId = (Long) searchResultItem.getValue();
                    array[j] = lexieId.longValue();
                    if (searchTextePositionArray != null) {
                        searchTextePositionArray[j] = searchResultItem.getSearchTextPositionInCollatedKey();
                    }
                }
                lexieIdArrays[i] = array;
            }

        }

        private boolean isInit() {
            return (lexieIdArrays != null);
        }

        /*
         * Retourne le nombre de départs possibles pour la séquence. Exemple, si la séquence est
         * "gouv* eau", les départs peuvent être gouvernement ou gouvernance.
         */
        private int getPotentialStartCount() {
            return lexieIdArrays[0].length;
        }

        /*
         * Indique que les tests (voir testLexification) vont se faire sur le i-ème départ possible
         * et retourne l'identifiant de ce  i-ème départ
         * (voir getPotentialStartCount ci-dessus)
         */
        private long setCurrentPotentialStart(int i) {
            currentPotentialStart = i;
            return lexieIdArrays[0][i];
        }

        public SubstringPosition testLexification(Lexification lexification, int index) {
            int length = lexieIdArrays.length;
            if ((index + length) > lexification.getCount()) {
                return null;
            }
            int lastTokenLexieIndex = -1;
            for (int i = 1; i < length; i++) {
                long lexieId = lexification.getLexieId(index + i);
                int test = test(lexieId, i);
                if (test == -1) {
                    return null;
                } else if (i == (length - 1)) {
                    lastTokenLexieIndex = test;
                }
            }
            SubstringPosition firstSubstringPosition = lexification.getCollationUnitPosition(index).convertToPositionInWholeSource(firstSearchTextePositionArray[currentPotentialStart]);
            SubstringPosition lastSubstringPosition = lexification.getCollationUnitPosition(index + length - 1).convertToPositionInWholeSource(lastSearchTextePositionArray[lastTokenLexieIndex]);
            SubstringPosition finalSubstringPosition = SubstringPosition.merge(firstSubstringPosition, lastSubstringPosition);
            return finalSubstringPosition;
        }

        private int test(long lexieId, int tokenIndex) {
            long[] potentialArray = lexieIdArrays[tokenIndex];
            int length = potentialArray.length;
            for (int i = 0; i < length; i++) {
                if (potentialArray[i] == lexieId) {
                    return i;
                }
            }
            return -1;
        }

    }

}
