/* ScrutariLib_SearchEngine - Copyright (c) 2010-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine.seeker;

import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.db.CodeFilter;
import net.mapeadores.scrutari.db.codefilter.CodeFilterUtils;
import net.mapeadores.scrutari.searchengine.SearchFilter;
import net.mapeadores.scrutari.searchengine.SearchOptions;
import net.mapeadores.util.text.TextConstants;


/**
 *
 * @author Vincent Calame
 */
public class SeekOptionsBuilder {

    private boolean avecMotscles;
    private boolean withFiches;
    private ScrutariDB scrutariDB;
    private DataAccess dataAccess;
    private SearchOptions searchOptions;

    public SeekOptionsBuilder(ScrutariDB scrutariDB, DataAccess dataAccess) {
        this.scrutariDB = scrutariDB;
        this.dataAccess = dataAccess;
    }

    public void setSearchOptions(SearchOptions searchOptions) {
        this.searchOptions = searchOptions;
    }

    public void setWithMotscles(boolean avecMotscles) {
        this.avecMotscles = avecMotscles;
    }

    public void setWithFiches(boolean withFiches) {
        this.withFiches = withFiches;
    }

    public SeekOptions toSeekOptions() {
        InternalSeekOptions seekOptions = new InternalSeekOptions();
        seekOptions.build();
        return seekOptions;
    }


    private class InternalSeekOptions implements SeekOptions {

        private InternalSearchFilter searchFilter;
        private int[] langIntegerArray;
        private int defaultSearchType = TextConstants.STARTSWITH;
        private boolean touteslangues = false;

        public int[] getLangIntegerArray() {
            return langIntegerArray;
        }

        public boolean isWithFiches() {
            return withFiches;
        }

        public boolean isWithMotscles() {
            return avecMotscles;
        }

        public int getDefaultSearchType() {
            return defaultSearchType;
        }

        public SearchFilter getSearchFilter() {
            return searchFilter;
        }

        public boolean isAllLang() {
            return touteslangues;
        }

        private void build() {
            searchFilter = new InternalSearchFilter();
            if (searchOptions != null) {
                searchFilter.build(searchOptions);
                defaultSearchType = searchOptions.getDefaultSearchType();
                int[] langIntArray = searchOptions.getLangIntegerArray();
                if (langIntArray != null) {
                    touteslangues = false;
                    this.langIntegerArray = langIntArray;
                }
            }
            if (langIntegerArray == null) {
                touteslangues = true;
                langIntegerArray = scrutariDB.getLangStats().getContentLangArray();
            }
        }

    }


    private class InternalSearchFilter implements SearchFilter {

        private CodeList codecorpusList;
        private CodeList codethesaurusList;
        private CodeFilter codeficheFilter = CodeFilterUtils.ALL_CODEFILTER;
        private CodeFilter codemotcleFilter = CodeFilterUtils.ALL_CODEFILTER;

        InternalSearchFilter() {
            codecorpusList = dataAccess.getCodecorpusList();
            codethesaurusList = dataAccess.getCodethesaurusList();
        }

        public CodeList getCodecorpusList() {
            return codecorpusList;
        }

        public CodeList getCodethesaurusList() {
            return codethesaurusList;

        }

        public CodeFilter getCodeficheFilter() {
            return codeficheFilter;
        }

        public CodeFilter getCodemotcleFilter() {
            return codemotcleFilter;
        }

        private void build(SearchOptions searchOptions) {
            CodeList optionCodecorpusList = searchOptions.getCodecorpusList();
            if (optionCodecorpusList != null) {
                this.codecorpusList = optionCodecorpusList;
            }
            CodeList optionCodethesaurusList = searchOptions.getCodethesaurusList();
            if (optionCodethesaurusList != null) {
                this.codethesaurusList = optionCodethesaurusList;
            }
            CodeFilter optionCodeficheFilter = searchOptions.getCodeficheFilter();
            if (optionCodeficheFilter != null) {
                this.codeficheFilter = optionCodeficheFilter;
            }
            CodeFilter optionCodemotcleFilter = searchOptions.getCodemotcleFilter();
            if (optionCodemotcleFilter != null) {
                this.codemotcleFilter = optionCodemotcleFilter;
            }
        }

    }

}
