/* SctServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.conf;

import net.scrutari.datauri.CorpusURI;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.SctServerLog;
import net.mapeadores.scrutari.searchengine.CorpusClasse;
import fr.exemole.sctserver.conf.xml.CorpusClassesDOMReader;
import net.mapeadores.util.logging.MutableProcessLog;
import net.mapeadores.util.logging.ProcessLog;
import net.mapeadores.util.logging.ProcessLogItem;
import net.mapeadores.util.logging.ProcessURI;
import net.mapeadores.util.text.DefaultLibelleHolder;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.scrutari.searchengine.PertinencePonderation;
import net.mapeadores.util.text.Libelle;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class CorpusClassesConf {

    private CorpusClassesLog processLog;
    private CorpusClasse[] corpusClasseArray;
    private ProcessURI processURI;
    private PertinencePonderation globalPertinencePonderation;

    private CorpusClassesConf() {
    }

    public ProcessLog getProcessLog() {
        return processLog;
    }

    public CorpusClasse[] getCorpusClasseArray() {
        return corpusClasseArray;
    }

    public PertinencePonderation getGlobalPertinencePonderation() {
        return globalPertinencePonderation;
    }

    public static CorpusClassesConf load(File f) throws IOException, ParserConfigurationException {
        CorpusClassesConf conf = new CorpusClassesConf();
        conf.init(f);
        return conf;
    }

    public ProcessURI getProcessURI() {
        return processURI;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        corpusClasseArray = null;
        processURI = ProcessURI.fromFile(f);
        processLog = new CorpusClassesLog(processURI);
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document = null;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            processLog.saxException(saxe);
            return;
        }
        CorpusClasseBuilder builder = new CorpusClasseBuilder();
        CorpusClassesDOMReader domReader = new CorpusClassesDOMReader(builder, processLog);
        domReader.readCorpusClasses(document.getDocumentElement());
        corpusClasseArray = builder.toCorpusClasseArray();
        globalPertinencePonderation = builder.globalPertinencePonderation;
    }


    private static class CorpusClasseBuilder implements CorpusClassesDOMReader.Builder {

        Map<String, InternalCorpusClasse> map = new LinkedHashMap<String, InternalCorpusClasse>();
        Map<CorpusURI, InternalCorpusClasse> uriMap = new HashMap<CorpusURI, InternalCorpusClasse>();
        PertinencePonderation globalPertinencePonderation;

        public CorpusClasse addClasse(String name) {
            InternalCorpusClasse corpusClasse = map.get(name);
            if (!map.containsKey(name)) {
                corpusClasse = new InternalCorpusClasse(name);
                map.put(name, corpusClasse);
            }
            return corpusClasse;
        }

        public CorpusClasse getCorpusClasse(String name) {
            return (CorpusClasse) map.get(name);
        }

        public void setGlobalPertinencePonderation(PertinencePonderation pertinencePonderation) {
            this.globalPertinencePonderation = pertinencePonderation;
        }

        public void putLibelle(CorpusClasse corpusClasse, Libelle libelle) {
            ((InternalCorpusClasse) corpusClasse).putLibelle(libelle);
        }

        public boolean addURI(CorpusClasse corpusClasse, CorpusURI corpusURI) {
            InternalCorpusClasse internalCorpusClasse = (InternalCorpusClasse) corpusClasse;
            if (uriMap.containsKey(corpusURI)) {
                return false;
            }
            uriMap.put(corpusURI, internalCorpusClasse);
            internalCorpusClasse.addURI(corpusURI);
            return true;
        }

        private void flush() {
            if (!map.containsKey(null)) {
                map.put(null, new InternalCorpusClasse(null));
            }
            int value = map.size();
            for (Map.Entry<String, InternalCorpusClasse> entry : map.entrySet()) {
                InternalCorpusClasse corpusClasse = entry.getValue();
                int rank = value * 10;
                value = value - 1;
                corpusClasse.setRank(rank);
            }
        }

        private InternalCorpusClasse[] toCorpusClasseArray() {
            int size = map.size();
            if (size == 0) {
                return null;
            }
            flush();
            InternalCorpusClasse[] result = new InternalCorpusClasse[size];
            int p = 0;
            for (InternalCorpusClasse icc : map.values()) {
                result[p] = icc;
                p++;
            }
            return result;
        }

    }


    private static class CorpusClassesLog extends MutableProcessLog implements CorpusClassesDOMReader.ErrorHandler {

        public CorpusClassesLog(ProcessURI processURI) {
            setCurrentURI(processURI);
        }

        public void saxException(SAXException saxe) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.MALFORMED_XML_CATEGORY);
            logItem.setKey(SctServerLog.SEVERE_DATA_BASEFILE_XML);
            logItem.setObject(saxe.getMessage());
        }

        public void unknownTagWarning(String tagName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("unknownTag");
            logItem.setObject(tagName);
        }

        public void emptyAttributeError(String tagName, String attributeName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("emptyAttribute");
            logItem.setObject(tagName + "@" + attributeName);
        }

        public void emptyElementError(String tagName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("emptyElement");
            logItem.setObject(tagName);
        }

        public void uriSyntaxError(String uriString) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("uriSyntaxError");
            logItem.setObject(uriString);
        }

        public void unknownCorpusClasseError(String corpusClasseName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("unknownCorpusClasse");
            logItem.setObject(corpusClasseName);
        }

    }


    private static class InternalCorpusClasse extends DefaultLibelleHolder implements CorpusClasse {

        private String name;
        private List<CorpusURI> uriList = new ArrayList<CorpusURI>();
        private int rank;

        private InternalCorpusClasse(String name) {
            super((name == null) ? "_default" : name);
            this.name = name;
        }

        public String getName() {
            return name;
        }

        public int getCorpusURICount() {
            return uriList.size();
        }

        public CorpusURI getCorpusURI(int i) {
            return uriList.get(i);
        }

        private void putLibelle(Libelle libelle) {
            innerPutLibelle(libelle);
        }

        private void addURI(CorpusURI corpusURI) {
            uriList.add(corpusURI);
        }

        public int getRank() {
            return rank;
        }

        private void setRank(int rank) {
            this.rank = rank;
        }

    }

}
