/* SctServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.conf;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.SctServerLog;
import fr.exemole.sctserver.feed.SctFeedMetadata;
import java.io.File;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.logging.MutableProcessLog;
import net.mapeadores.util.logging.ProcessLog;
import net.mapeadores.util.logging.ProcessLogItem;
import net.mapeadores.util.logging.ProcessURI;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class FeedConf {

    private FeedLog feedLog;
    private ProcessURI processURI;
    private SctFeedMetadata sctFeedMetadata;

    public static FeedConf load(File f) throws IOException, ParserConfigurationException {
        FeedConf conf = new FeedConf();
        conf.init(f);
        return conf;
    }

    public ProcessLog getProcessLog() {
        return feedLog;
    }

    public ProcessURI getProcessURI() {
        return processURI;
    }

    public SctFeedMetadata getSctFeedMetadata() {
        return sctFeedMetadata;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        processURI = ProcessURI.fromFile(f);
        feedLog = new FeedLog(processURI);
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document = null;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            feedLog.saxException(saxe);
            return;
        }
        /*UriCodesBuilder builder = new UriCodesBuilder(uriCodesLog);
        UriCodesDOMReader domReader = new UriCodesDOMReader(builder, feedLog);
        domReader.readUriCodes(document.getDocumentElement());*/
    }


    private static class FeedLog extends MutableProcessLog /*implements UriCodesDOMReader.ErrorHandler*/ {

        public FeedLog(ProcessURI processURI) {
            setCurrentURI(processURI);
        }

        public void saxException(SAXException saxe) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.MALFORMED_XML_CATEGORY);
            logItem.setKey(SctServerLog.SEVERE_DATA_BASEFILE_XML);
            logItem.setObject(saxe.getMessage());
        }

    }

}
