/* SctServer - Copyright (c) 2009 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.conf;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.SctServerLog;
import fr.exemole.sctserver.conf.xml.SourcesDOMReader;
import net.mapeadores.util.logging.MutableProcessLog;
import net.mapeadores.util.logging.ProcessLog;
import net.mapeadores.util.logging.ProcessLogItem;
import net.mapeadores.util.logging.ProcessURI;
import net.mapeadores.util.text.UrlKey;
import java.io.File;
import java.io.IOException;
import java.util.Set;
import java.util.LinkedHashSet;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;


/**
 *
 * @author Vincent Calame
 */
public class SourcesConf {

    private SourcesLog processLog;
    private UrlKey[] infoUrlKeyArray = null;
    private ProcessURI processURI;

    public SourcesConf() {
    }

    public ProcessLog getProcessLog() {
        return processLog;
    }

    public ProcessURI getProcessURI() {
        return processURI;
    }

    public UrlKey[] getInfoUrlKeyArray() {
        return infoUrlKeyArray;
    }

    public static SourcesConf load(File f) throws IOException, ParserConfigurationException {
        SourcesConf conf = new SourcesConf();
        conf.init(f);
        return conf;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        infoUrlKeyArray = null;
        processURI = ProcessURI.fromFile(f);
        processLog = new SourcesLog(processURI);
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document = null;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            processLog.saxException(saxe);
            return;
        }
        SourcesBuilder builder = new SourcesBuilder();
        SourcesDOMReader domReader = new SourcesDOMReader(builder, processLog);
        domReader.readSources(document.getDocumentElement());
        infoUrlKeyArray = builder.toInfoUrlKeyArray();
    }


    private static class SourcesBuilder implements SourcesDOMReader.Builder {

        Set<UrlKey> set = new LinkedHashSet<UrlKey>();

        public boolean addInfoUrl(UrlKey urlKey) {
            if (set.contains(urlKey)) {
                return false;
            }
            set.add(urlKey);
            return true;
        }

        private UrlKey[] toInfoUrlKeyArray() {
            int size = set.size();
            if (size == 0) {
                return null;
            }
            UrlKey[] infoUrlKeyArray = new UrlKey[size];
            int p = 0;
            for (UrlKey urlKey : set) {
                infoUrlKeyArray[p] = urlKey;
                p++;
            }
            return infoUrlKeyArray;
        }

    }


    private static class SourcesLog extends MutableProcessLog implements SourcesDOMReader.ErrorHandler {

        public SourcesLog(ProcessURI processURI) {
            setCurrentURI(processURI);
        }

        public void saxException(SAXException saxe) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.MALFORMED_XML_CATEGORY);
            logItem.setKey(SctServerLog.SEVERE_DATA_BASEFILE_XML);
            logItem.setObject(saxe.getMessage());
        }

        public void unknownTagWarning(String tagName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("unknownTag");
            logItem.setObject(tagName);
        }

        public void emptyElementError(String tagName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("emptyElement");
            logItem.setObject(tagName);
        }

        public void malformedUrlError(String uriString, String message) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("malformedUrl");
            logItem.setObject(uriString + " (" + message + ")");
        }

        public void doublonUrlWarning(String urlString, String urlKey) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("doublonUrl");
            logItem.setObject(urlString + " (" + urlKey + ")");
        }

    }

}
