/* SctServer - Copyright (c) 2009 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.conf.xml;

import net.mapeadores.util.text.UrlKey;
import net.mapeadores.util.xml.XMLUtils;
import java.net.MalformedURLException;
import java.net.URL;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class SourcesDOMReader {

    private ErrorHandler errorHandler;
    private Builder builder;

    public SourcesDOMReader(Builder builder, ErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.builder = builder;
    }

    public void readSources(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("info-url")) {
                    addInfoUrl(element);
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }

    public void addInfoUrl(Element element_xml) {
        String urlString = XMLUtils.getData(element_xml);
        if (urlString.length() == 0) {
            errorHandler.emptyElementError("info-url");
            return;
        }
        try {
            URL url = new URL(urlString);
            UrlKey urlKey = new UrlKey(url);
            boolean done = builder.addInfoUrl(urlKey);
            if (!done) {
                errorHandler.doublonUrlWarning(urlString, urlKey.getKeyString());
            }
        } catch (MalformedURLException mue) {
            errorHandler.malformedUrlError(urlString, mue.getMessage());
        }
    }


    public static interface ErrorHandler {

        public void unknownTagWarning(String tagName);

        public void emptyElementError(String tagName);

        public void malformedUrlError(String urlString, String message);

        public void doublonUrlWarning(String urlString, String urlKey);

    }


    public static interface Builder {

        public boolean addInfoUrl(UrlKey urlKey);

    }

}
