/* SctServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.conf.xml;

import net.mapeadores.util.xml.XMLUtils;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 *
 * @author Vincent Calame
 */
public class UriCodesDOMReader {

    private ErrorHandler errorHandler;
    private Builder builder;

    public UriCodesDOMReader(Builder builder, ErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.builder = builder;
    }

    public void readUriCodes(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("uri")) {
                    addUri(element);
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }

    private void addUri(Element element_xml) {
        String codeString = element_xml.getAttribute("code");
        if (codeString.length() == 0) {
            errorHandler.emptyAttributeError("uri", "code");
            return;
        }
        int code = 0;
        try {
            code = Integer.parseInt(codeString);
            if (code < 1) {
                errorHandler.notPositiveIntegerAttributeError("uri", "code", codeString);
            }
        } catch(NumberFormatException nfe) {
            errorHandler.notPositiveIntegerAttributeError("uri", "code", codeString);
        }
        String uriString = XMLUtils.getData(element_xml);
        if (uriString.length() == 0) {
            errorHandler.emptyElementError("uri");
            return;
        }
        try {
            ScrutariDataURI scrutariDataURI = URIParser.parse(uriString);
            builder.addScrutariDataURI(code, scrutariDataURI);
        } catch (URIParseException uripe) {
            errorHandler.uriSyntaxError(uriString);
            return;
        }
    }

    public static interface ErrorHandler {

        public void unknownTagWarning(String tagName);

        public void emptyAttributeError(String tagName, String attributeName);

        public void notPositiveIntegerAttributeError(String tagName, String attributeName, String attributeValue);

        public void emptyElementError(String tagName);

        public void uriSyntaxError(String uriString);

    }


    public static interface Builder {

        public void addScrutariDataURI(int code, ScrutariDataURI scrutariDataURI);

    }
}
