/* SctServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.feed;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.storage.SctServerStorage;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import net.mapeadores.scrutari.db.ScrutariDBName;


/**
 *
 * @author Vincent Calame
 */
public class SctFeedBuilder {

    private InternalSctFeed sctFeed = new InternalSctFeed();
    private int entryMax = 10;
    private DateFormat dayFormat = new SimpleDateFormat("yyyyMMdd");

    public SctFeedBuilder() {
    }

    public void build(SctServer sctServer) {
        SctServerStorage sctServerStorage = sctServer.getSctServerStorage();
        sctFeed.setMetadata(sctServerStorage.getConfDirectory().getSctFeedMetadata());
        Date[] dateArray = sctServerStorage.getAddURITreeDateArray();
        Date lastDate = null;
        int length = dateArray.length;
        if (length == 0) {
            ScrutariDBName scrutariDBName = sctServer.getScrutariDB().getScrutariDBName();
            lastDate = scrutariDBName.getDate();
        } else {
            lastDate = dateArray[0];
        }
        sctFeed.setLastDate(lastDate);
        if (length > 0) {
            int step = 1;
            String previousDayString = null;
            for (int i = 0; i < length; i++) {
                Date date = dateArray[i];
                String dayString = dayFormat.format(date);
                if (i > 0) {
                    if (!dayString.equals(previousDayString)) {
                        step++;
                    }
                }
                previousDayString = dayString;
                InternalSctFeedEntry entry = new InternalSctFeedEntry(date);
                sctFeed.add(entry);
                if (step == entryMax) {
                    break;
                }
            }
        }
    }

    public SctFeed toSctFeed() {
        return sctFeed;
    }


    private class InternalSctFeed implements SctFeed {

        private long lastModified;
        private SctFeedMetadata sctFeedMetadata;
        private String feedUpdated;
        private List<InternalSctFeedEntry> entryList = new ArrayList<InternalSctFeedEntry>();

        public long getLastModified() {
            return lastModified;
        }

        public SctFeedMetadata getMetadata() {
            return sctFeedMetadata;
        }

        public String getFeedUpdated() {
            return feedUpdated;
        }

        public int getSctFeedEntryCount() {
            return entryList.size();
        }

        public SctFeedEntry getSctFeedEntry(int i) {
            return entryList.get(i);
        }

        private void setMetadata(SctFeedMetadata sctFeedMetadata) {
            this.sctFeedMetadata = sctFeedMetadata;
        }

        private void setLastDate(Date date) {
            this.lastModified = date.getTime();
            this.feedUpdated = ScrutariDBName.toGmtIsoFormat(date);
        }

        private void add(InternalSctFeedEntry entry) {
            entryList.add(entry);
        }

    }


    private static class InternalSctFeedEntry implements SctFeedEntry {

        private Date date;

        private InternalSctFeedEntry(Date date) {
            this.date = date;
        }

        public Date getDate() {
            return date;
        }

    }

}
