/* SctServer - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import net.scrutari.datauri.FicheURI;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.lexie.FieldNumbers;
import net.mapeadores.scrutari.searchengine.FicheSearchResultInfo;
import net.mapeadores.scrutari.searchengine.SearchTokenPositionList;
import fr.exemole.sctserver.request.FicheFieldsFilter;
import fr.exemole.sctserver.request.MotcleFieldsFilter;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.searchengine.MotcleSearchResultInfo;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.SubstringPosition;
import net.scrutari.datauri.MotcleURI;


/**
 *
 * @author Vincent Calame
 */
public class JsonUtils implements FieldNumbers {

    public static void writeMLib(JSONWriter jsonWriter, String lib, SearchTokenPositionList positionList) throws JSONException {
        PositionListBuilder builder = new PositionListBuilder(jsonWriter, lib, positionList);
        builder.buildArray();
    }

    public static void writeMLib(JSONWriter jsonWriter, String lib) throws JSONException {
        jsonWriter.array();
        jsonWriter.value(lib);
        jsonWriter.endArray();
    }

    public static void addMotcleData(JSONWriter jsonWriter, MotcleData motcleData, MotcleFieldsFilter motcleFieldsFilter, MotcleSearchResultInfo motcleSearchResultInfo) throws JSONException {
        jsonWriter.key("codemotcle");
        jsonWriter.value(motcleData.getCodeMotcle());
        if (motcleFieldsFilter.isWithCodeThesaurus()) {
            jsonWriter.key("codethesaurus");
            jsonWriter.value(motcleData.getCodeThesaurus());
        }
        if (motcleFieldsFilter.isWithCodeBase()) {
            jsonWriter.key("codebase");
            jsonWriter.value(motcleData.getCodeBase());
        }
        if (motcleFieldsFilter.isWithBaseName()) {
            MotcleURI motcleURI = motcleData.getMotcleURI();
            jsonWriter.key("basename");
            jsonWriter.value(motcleURI.getThesaurusURI().getBaseURI().getBaseName());
        }
        if (motcleFieldsFilter.isWithThesaurusName()) {
            MotcleURI motcleURI = motcleData.getMotcleURI();
            jsonWriter.key("thesaurusname");
            jsonWriter.value(motcleURI.getThesaurusURI().getThesaurusName());
        }
        if (motcleFieldsFilter.isWithMotcleId()) {
            MotcleURI motcleURI = motcleData.getMotcleURI();
            jsonWriter.key("motcleid");
            jsonWriter.value(motcleURI.getMotcleId());
        }
        if (motcleFieldsFilter.isWithLibelles()) {
            int libcount = motcleData.getLibelleCount();
            if (libcount > 0) {
                jsonWriter.key("libelleArray");
                jsonWriter.array();
                for (int i = 0; i < libcount; i++) {
                    jsonWriter.object();
                    Libelle lib = motcleData.getLibelleByIndex(i);
                    jsonWriter.key("lang");
                    jsonWriter.value(LangInteger.toString(lib.getLangInteger()));
                    jsonWriter.key("lib");
                    jsonWriter.value(lib.getLibelleString());
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
        }
        if (motcleFieldsFilter.isWithMLibelles()) {
            int libcount = motcleData.getLibelleCount();
            if (libcount > 0) {
                boolean withDefault = motcleFieldsFilter.isWithLibellesDefault();
                boolean inclus = false;
                for (int i = 0; i < libcount; i++) {
                    Libelle lib = motcleData.getLibelleByIndex(i);
                    int langInteger = lib.getLangInteger();
                    SearchTokenPositionList positionList = motcleSearchResultInfo.getSearchTokentPositionListByLang(langInteger);
                    if ((!withDefault) && (positionList == null)) {
                        continue;
                    }
                    if (!inclus) {
                        jsonWriter.key("mlibelleArray");
                        jsonWriter.array();
                        inclus = true;
                    }
                    jsonWriter.object();
                    jsonWriter.key("lang");
                    jsonWriter.value(LangInteger.toString(langInteger));
                    jsonWriter.key("mlib");
                    if (positionList != null) {
                        writeMLib(jsonWriter, lib.getLibelleString(), positionList);
                    } else {
                        writeMLib(jsonWriter, lib.getLibelleString());
                    }
                    jsonWriter.endObject();
                }
                if (inclus) {
                    jsonWriter.endArray();
                    inclus = true;
                }
            }
        }
    }

    public static void addFicheData(JSONWriter jsonWriter, FicheData ficheData, FicheFieldsFilter ficheFieldsFilter, int langUi, FicheSearchResultInfo info) throws JSONException {
        jsonWriter.key("codefiche");
        jsonWriter.value(ficheData.getCodeFiche());
        if (ficheFieldsFilter.isWithCodeCorpus()) {
            jsonWriter.key("codecorpus");
            jsonWriter.value(ficheData.getCodeCorpus());
        }
        if (ficheFieldsFilter.isWithCodeBase()) {
            jsonWriter.key("codebase");
            jsonWriter.value(ficheData.getCodeBase());
        }
        if (ficheFieldsFilter.isWithBaseName()) {
            FicheURI ficheURI = ficheData.getFicheURI();
            jsonWriter.key("basename");
            jsonWriter.value(ficheURI.getCorpusURI().getBaseURI().getBaseName());
        }
        if (ficheFieldsFilter.isWithCorpusName()) {
            FicheURI ficheURI = ficheData.getFicheURI();
            jsonWriter.key("corpusname");
            jsonWriter.value(ficheURI.getCorpusURI().getCorpusName());
        }
        if (ficheFieldsFilter.isWithFicheId()) {
            FicheURI ficheURI = ficheData.getFicheURI();
            jsonWriter.key("ficheid");
            jsonWriter.value(ficheURI.getFicheId());
        }
        if (ficheFieldsFilter.isWithTitre()) {
            String titre = ficheData.getTitre();
            if (titre != null) {
                jsonWriter.key("titre");
                jsonWriter.value(ficheData.getTitre());
            }
        }
        if (ficheFieldsFilter.isWithSoustitre()) {
            String soustitre = ficheData.getSoustitre();
            if (soustitre != null) {
                jsonWriter.key("soustitre");
                jsonWriter.value(soustitre);
            }
        }
        if (ficheFieldsFilter.isWithHref()) {
            String href = ficheData.getHref(langUi);
            if (href != null) {
                jsonWriter.key("href");
                jsonWriter.value(href);
            }
        }
        if (ficheFieldsFilter.isWithLang()) {
            int lang = ficheData.getLangInteger();
            if (lang != LangInteger.UNDETERMINED) {
                jsonWriter.key("lang");
                jsonWriter.value(LangInteger.toString(lang));
            }
        }
        if (ficheFieldsFilter.isWithAnnee()) {
            int dateInteger = ficheData.getDateInteger();
            if (dateInteger != DateInteger.NULL) {
                jsonWriter.key("annee");
                jsonWriter.value(DateInteger.getAnnee(dateInteger));
            }
        }
        if (ficheFieldsFilter.isWithFicheIcon()) {
            String ficheIcon = ficheData.getFicheIcon();
            if (ficheIcon != null) {
                jsonWriter.key("ficheicon");
                jsonWriter.value(ficheIcon);
            }
        }
        if (ficheFieldsFilter.isWithGeo()) {
            String latitude = ficheData.getLatitude();
            if (latitude != null) {
                jsonWriter.key("lat");
                jsonWriter.value(latitude);
                jsonWriter.key("lon");
                jsonWriter.value(ficheData.getLongitude());
            }
        }
        if (ficheFieldsFilter.isWithComplements()) {
            int maxNumber = ficheData.getComplementMaxNumber();
            if (maxNumber > 0) {
                boolean inclus = false;
                for (int num = 1; num <= maxNumber; num++) {
                    String comp = ficheData.getComplementByNumber(num);
                    if (comp == null) {
                        continue;
                    }
                    if (!inclus) {
                        jsonWriter.key("complementArray");
                        jsonWriter.array();
                        inclus = true;
                    }
                    jsonWriter.object();
                    jsonWriter.key("num");
                    jsonWriter.value(num);
                    jsonWriter.key("comp");
                    jsonWriter.value(comp);
                    jsonWriter.endObject();
                }
                if (inclus) {
                    jsonWriter.endArray();
                }
            }
        }
        if (info == null) {
            return;
        }
        if (ficheFieldsFilter.isWithMTitre()) {
            String titre = ficheData.getTitre();
            if (titre != null) {
                SearchTokenPositionList positionList = info.getSubstringPositionList(TITRE_FIELDNUMBER);
                if (positionList != null) {
                    jsonWriter.key("mtitre");
                    writeMLib(jsonWriter, titre, positionList);
                } else {
                    if (ficheFieldsFilter.isWithTitreDefault()) {
                        jsonWriter.key("mtitre");
                        writeMLib(jsonWriter, titre);
                    }
                }
            }
        }
        if (ficheFieldsFilter.isWithMSoustitre()) {
            String soustitre = ficheData.getSoustitre();
            if (soustitre != null) {
                SearchTokenPositionList positionList = info.getSubstringPositionList(SOUSTITRE_FIELDNUMBER);
                if (positionList != null) {
                    jsonWriter.key("msoustitre");
                    writeMLib(jsonWriter, soustitre, positionList);
                } else {
                    if (ficheFieldsFilter.isWithSoustitreDefault()) {
                        jsonWriter.key("msoustitre");
                        writeMLib(jsonWriter, soustitre);
                    }
                }
            }
        }
        if (ficheFieldsFilter.isWithMComplements()) {
            boolean withDefault = ficheFieldsFilter.isWithComplementsDefault();
            int maxNumber = ficheData.getComplementMaxNumber();
            if (maxNumber > 0) {
                boolean inclus = false;
                int codecorpus = ficheData.getCodeCorpus();
                for (int num = 1; num <= maxNumber; num++) {
                    String comp = ficheData.getComplementByNumber(num);
                    if (comp == null) {
                        continue;
                    }
                    SearchTokenPositionList positionList = info.getSubstringPositionList(num);
                    if ((positionList == null) && (!withDefault)) {
                        continue;
                    }
                    if (!inclus) {
                        jsonWriter.key("mcomplementArray");
                        jsonWriter.array();
                        inclus = true;
                    }
                    jsonWriter.object();
                    jsonWriter.key("num");
                    jsonWriter.value(num);
                    jsonWriter.key("mcomp");
                    if (positionList != null) {
                        writeMLib(jsonWriter, comp, positionList);
                    } else {
                        writeMLib(jsonWriter, comp);
                    }
                    jsonWriter.endObject();
                }
                if (inclus) {
                    jsonWriter.endArray();
                }
            }
        }
        if (ficheFieldsFilter.isWithPertinence()) {
            jsonWriter.key("pertinence");
            jsonWriter.value(info.getPertinence());
        }
    }


    private static class PositionListBuilder {

        private JSONWriter jsonWriter;
        SearchTokenPositionList positionList;
        StringBuilder buf;
        String s;
        int openIndex;
        int closeIndex;
        int currentPosition;

        /*
         * Constructeur unique.
         */
        PositionListBuilder(JSONWriter jsonWriter, String s, SearchTokenPositionList positionList) {
            this.jsonWriter = jsonWriter;
            this.positionList = positionList;
            this.s = s;
            currentPosition = 0;
            openIndex = positionList.getSearchTokenPosition(0).getBeginIndex();
            closeIndex = positionList.getSearchTokenPosition(0).getEndIndex();
            buf = new StringBuilder();
        }

        /*
         * Lancement de l'écriture.
         */
        void buildArray() throws JSONException {
            jsonWriter.array();
            int length = s.length();
            boolean onObject = false;
            for (int i = 0; i < length; i++) {
                if (i == openIndex) {
                    flushBuffer(false);
                    jsonWriter.object();
                    jsonWriter.key("i");
                    jsonWriter.value(positionList.getSearchTokenIndexForPosition(currentPosition) + 1);
                    jsonWriter.key("s");
                    onObject = true;
                }
                char carac = s.charAt(i);
                buf.append(carac);
                if (i == closeIndex) {
                    flushBuffer(true);
                    jsonWriter.endObject();
                    increment();
                    onObject = false;
                }
            }
            flushBuffer(false);
            if (onObject) {
                jsonWriter.endObject();
            }
            jsonWriter.endArray();
        }

        void flushBuffer(boolean withEmpty) throws JSONException {
            String s = buf.toString();
            if ((!withEmpty) && (s.length() == 0)) {
                return;
            }
            jsonWriter.value(buf.toString());
            buf = new StringBuilder();
        }

        /*
         * gère les éventuels recouvrements de position. Cela peut aller jusqu'à générer un élément s vide (<s token-index=""/>)
         * si une position est complètement incluse dans une autre.
         */
        void increment() throws JSONException {
            if (currentPosition == (positionList.size() - 1)) {
                openIndex = -1;
                closeIndex = -1;
                return;
            }
            currentPosition++;
            SubstringPosition sp = positionList.getSearchTokenPosition(currentPosition);
            if (sp.getBeginIndex() > closeIndex) {
                openIndex = sp.getBeginIndex();
                closeIndex = sp.getEndIndex();
                return;
            }
            openIndex = -1;
            jsonWriter.object();
            jsonWriter.key("i");
            jsonWriter.value(positionList.getSearchTokenIndexForPosition(currentPosition) + 1);
            if (sp.getEndIndex() <= closeIndex) {
                jsonWriter.endObject();
                increment();
            } else {
                jsonWriter.key("s");
                closeIndex = sp.getEndIndex();
            }
        }

    }

}
