/* SctServer - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.xml;

import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.searchengine.FicheSearchResult;
import net.mapeadores.scrutari.searchengine.MotcleSearchResultInfo;
import net.mapeadores.scrutari.searchengine.SearchTokenPositionList;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.config.ConfigException;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.xml.XMLUtils;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.scrutari.db.ScrutariDB;


/**
 * Classe produisant l'arbre XML des mots-clés.
 *
 * @author Vincent Calame
 */
public class MotsclesRequest {

    private FicheSearchResult searchResult;
    private ScrutariDB scrutariDB;

    /**
     * Construit une nouvelle instance de MotsclesRequest à partir de la recherche d'identifiant searchId.
     */
    public MotsclesRequest(String searchid, SctServer sctServer) throws XmlRequestException {
        if (searchid == null) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.MISSING_SEARCHID_PARAMETER);
        }
        try {
            searchResult = sctServer.getSearchResult(searchid);
        } catch (ConfigException ce) {
            if (ce.hasSource()) {
                throw new XmlRequestException(ce.getSource());
            } else {
                throw new XmlRequestException(ce);
            }
        }
        if (searchResult == null) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.WRONG_SEARCHID_PARAMETER, searchid);
        }
        this.scrutariDB = sctServer.getScrutariDB();
    }

    /**
     * Écrit l'arbre XML des mots-clés comme réponse à une requête de servlet.
     */
    public void printResponse(HttpServletResponse reponse) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        int tabulations = 0;
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8' standalone='no' ?>");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<!DOCTYPE motscles SYSTEM \"dtd/motscles.dtd\">");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<motscles id=\"motscles\" search-id=\"");
        pw.print(searchResult.getIdentifiant());
        /*pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(searchResult.getFicheCount()));
        pw.print("\" fiche-maximum=\"");
        pw.print(String.valueOf(searchResult.getFicheMaximum()));*/
        pw.print("\" all-motcle-count=\"");
        pw.print(String.valueOf(searchResult.getMotcleCount()));
        /*pw.print("\" complete-motcle-count=\"");
        pw.print(String.valueOf(searchResult.getCompleteMotcleCount()));
        pw.print("\" partial-motcle-count=\"");
        pw.print(String.valueOf(searchResult.getPartialMotcleCount()));*/
        pw.print("\"");
        pw.print(">");
        int count = searchResult.getMotcleCount();
        for (int i = 0; i < count; i++) {
            printMotcleResult(pw, searchResult.getMotcleResult(i), tabulations + 1, dataAccess);
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</motscles>");
        dataAccess.close();
    }

    /*
     * Écrit les Datarmations liées à un mot-clé d'un résultat. Méthode appelée par printResponse.
     */
    private void printMotcleResult(PrintWriter pw, MotcleSearchResultInfo motcleResult, int tabulations, DataAccess dataAccess) {
        int code = motcleResult.getCode();
        int ficheCount = motcleResult.getAcceptedFicheCount();
        MotcleData motcleData = dataAccess.getMotcleDataByCode(code);
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<motcle id=\"");
        pw.print(String.valueOf(code));
        pw.print("\" motcle-code=\"");
        pw.print(String.valueOf(code));
        pw.print("\" thesaurus-code=\"");
        pw.print(String.valueOf(motcleData.getCodeThesaurus()));
        pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(ficheCount));
        pw.print("\" fiche-maximum=\"");
        pw.print(String.valueOf(motcleResult.getFicheMaximum()));
        pw.print("\"");
        pw.print(">");
        int libcount = motcleData.getLibelleCount();
        for (int i = 0; i < libcount; i++) {
            Libelle libelle = motcleData.getLibelleByIndex(i);
            int langInteger = libelle.getLangInteger();
            String strg_libelle = libelle.getLibelleString();
            SearchTokenPositionList positionList = motcleResult.getSearchTokentPositionListByLang(langInteger);
            String strg_lang = XmlRequestUtils.getLangString(langInteger);
            XMLUtils.printTabs(pw, tabulations + 1);
            pw.print("<lib id=\"");
            pw.print(String.valueOf(code));
            pw.print("_");
            pw.print(strg_lang);
            pw.print("\" lang=\"");
            pw.print(strg_lang);
            pw.print("\" role=\"");
            if (positionList == null) {
                pw.print("0");
            } else {
                pw.print("1");
            }
            pw.print("\">");
            XmlRequestUtils.printPositionList(pw, strg_libelle, positionList);
            XMLUtils.printTabs(pw, tabulations + 1);
            pw.print("</lib>");
        }

        for (int i = 0; i < ficheCount; i++) {
            int codeFiche = motcleResult.getAcceptedFicheCode(i);
            FicheData ficheData = dataAccess.getFicheDataByCode(codeFiche);
            XMLUtils.printTabs(pw, tabulations + 1);
            pw.print("<fch fiche-code=\"");
            pw.print(String.valueOf(codeFiche));
            int dateInteger = ficheData.getDateInteger();
            if (dateInteger != DateInteger.NULL) {
                pw.print("\" date=\"");
                pw.print(DateInteger.toString(dateInteger));
            }
            pw.print("\"/>");
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</motcle>");
    }

}
