/* SctServer - Copyright (c) 2005 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.xml;

import net.mapeadores.util.xml.XMLUtils;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.http.HttpServletResponse;
import org.xml.sax.SAXParseException;


/**
 *
 * @author Vincent Calame
 */
public class XmlRequestException extends Exception {

    public static final short RCHF_SOURCE = 1;
    public static final short CLIENT_SOURCE = 2;
    public static final String MISSING_CMD_PARAMETER = "MissingCmdParameter";
    public static final String MISSING_XML_PARAMETER = "MissingXmlParameter";
    public static final String MISSING_SEARCHID_PARAMETER = "MissingSearchidParameter";
    public static final String WRONG_CMD_PARAMETER = "WrongCmdParameter";
    public static final String WRONG_SEARCHID_PARAMETER = "WrongSearchidParameter";
    public static final String XML_PARAMETER_PARSE = "XmlParameterParse";
    public static final String MISSING_SOURCE_ELEMENT = "MissingSourceElement";
    public static final String POOR_SOURCE_ELEMENT = "PoorSourceElement";
    public static final String EXCEPTION = "Exception";
    private short sourceType;
    private String errorType;
    private String errorDetail;
    private Exception e;

    public XmlRequestException(short sourceType, String errorType) {
        super(errorType);
        this.errorType = errorType;
        this.sourceType = sourceType;
    }

    public XmlRequestException(short sourceType, String errorType, String errorDetail) {
        super(errorType);
        this.errorType = errorType;
        this.errorDetail = errorDetail;
        this.sourceType = sourceType;
    }

    public XmlRequestException(Exception e) {
        super(e);
        errorType = EXCEPTION;
        sourceType = RCHF_SOURCE;
        errorDetail = e.getClass().getName();
    }

    public XmlRequestException(SAXParseException saxPe) {
        super(XML_PARAMETER_PARSE);
        sourceType = CLIENT_SOURCE;
        errorType = XML_PARAMETER_PARSE;
        StringBuilder buf = new StringBuilder();
        buf.append(saxPe.getLocalizedMessage());
        buf.append(" ");
        String publicId = saxPe.getPublicId();
        if (publicId != null) {
            buf.append("publicId=");
            buf.append(publicId);
            buf.append(" ");
        }
        String systemId = saxPe.getSystemId();
        if (systemId != null) {
            buf.append("systemId=");
            buf.append(systemId);
            buf.append(" ");
        }
        buf.append("line=");
        buf.append(saxPe.getLineNumber());
        buf.append(" ");
        buf.append("column=");
        buf.append(saxPe.getColumnNumber());
        errorDetail = buf.toString();
    }

    public void printErrorResponse(HttpServletResponse reponse) throws IOException {
        int tabulations = 0;
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8' standalone='no' ?>");
        pw.print("<!DOCTYPE error SYSTEM \"dtd/error.dtd\">");
        pw.print("<error source=\"");
        switch (sourceType) {
            case RCHF_SOURCE:
                pw.print("rchf");
                break;
            case CLIENT_SOURCE:
                pw.print("client");
                break;
        }
        pw.print("\" type=\"");
        pw.print(errorType);
        if (errorDetail != null) {
            pw.print("\" detail=\"");
            XMLUtils.printEscape(pw, errorDetail);
        }
        pw.print("\"/>");
    }

}
