/* SctServer - Copyright (c) 2005-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.xml;

import net.mapeadores.scrutari.searchengine.SearchTokenPositionList;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.SubstringPosition;
import net.mapeadores.util.xml.XMLException;
import net.mapeadores.util.xml.XMLUtils;
import java.io.PrintWriter;
import org.w3c.dom.Document;


/**
 * Série de méthodes statiques utilisées pour l'écriture et la lecture des arbres Xml.
 *
 * @author Vincent Calame
 */
public class XmlRequestUtils {

    /**
     * Parse la chaîne xml en instance de org.w3c.dom.Document.
     */
    public static Document getDocument(String xml) throws XmlRequestException {
        try {
            return XMLUtils.getDocument(xml);
        } catch (XMLException xe) {
            throw new XmlRequestException(xe.getException());
        }
    }

    /**
     * Écrit la chaîne s en insérant des éléments <code>&lt;s token-index=""&gt;</code> suivant les positions
     * contenues dans l'instance de SearchResult.SubstringPositionList. Cette liste ne doit pas contenir de
     * positions qui vont au delà de la taille de la chaîne <code>s</code>.
     */
    public static void printPositionList(PrintWriter pw, String s, SearchTokenPositionList positionList) {
        if (positionList == null) {
            XMLUtils.printEscape(pw, s);
            return;
        }
        PositionListWriter writer = new PositionListWriter(pw, s, positionList);
        writer.print();
    }

    /**
     * Retourne la chaîne représentant la langue. Si (langInteger == IntLang.CODELANGUE_NULL), retourne "x-unknown" sinon retourne le code ISO.
     */
    public static String getLangString(int langInteger) {
        return (langInteger == LangInteger.UNDETERMINED) ? "x-unknown" : LangInteger.toString(langInteger, '-');
    }

    /*
     * Classe utilisée par la méthode printPositionList.
     */

    private static class PositionListWriter {

        SearchTokenPositionList positionList;
        PrintWriter pw;
        String s;
        int openIndex;
        int closeIndex;
        int currentPosition;

        /*
         * Constructeur unique.
         */
        PositionListWriter(PrintWriter pw, String s, SearchTokenPositionList positionList) {
            this.pw = pw;
            this.positionList = positionList;
            this.s = s;
            currentPosition = 0;
            openIndex = positionList.getSearchTokenPosition(0).getBeginIndex();
            closeIndex = positionList.getSearchTokenPosition(0).getEndIndex();
        }

        /*
         * Lancement de l'écriture.
         */
        void print() {
            int length = s.length();
            for (int i = 0; i < length; i++) {
                if (i == openIndex) {
                    pw.print("<s token-index=\"");
                    pw.print(String.valueOf(positionList.getSearchTokenIndexForPosition(currentPosition)));
                    pw.print("\">");
                }
                char carac = s.charAt(i);
                XMLUtils.printEscape(pw, carac);
                if (i == closeIndex) {
                    pw.print("</s>");
                    increment();
                }
            }
        }

        /*
         * gère les éventuels recouvrements de position. Cela peut aller jusqu'à générer un élément s vide (<s token-index=""/>)
         * si une position est complètement incluse dans une autre.
         */
        void increment() {
            if (currentPosition == (positionList.size() - 1)) {
                openIndex = -1;
                closeIndex = -1;
                return;
            }
            currentPosition++;
            SubstringPosition sp = positionList.getSearchTokenPosition(currentPosition);
            if (sp.getBeginIndex() > closeIndex) {
                openIndex = sp.getBeginIndex();
                closeIndex = sp.getEndIndex();
                return;
            }
            openIndex = -1;
            pw.print("<s token-index=\"");
            pw.print(String.valueOf(positionList.getSearchTokenIndexForPosition(currentPosition)));
            if (sp.getEndIndex() <= closeIndex) {
                pw.print("\"/>");
                increment();
            } else {
                pw.print("\">");
                closeIndex = sp.getEndIndex();
            }
        }

    }

}
