/* SctServer_Directory - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.directory.cache;

import fr.exemole.sctserver.storage.SctStorageException;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.HashMap;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.db.DataCacheReader;
import net.mapeadores.scrutari.db.DataCacheReaderFactory;
import net.mapeadores.scrutari.db.DataCacheWriter;
import net.mapeadores.util.primitives.io.DataInputPrimitivesReader;
import net.mapeadores.util.primitives.io.DataOutputPrimitivesWriter;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryDataCacheWriter implements DataCacheWriter {

    private HashMap<Integer, Long> positionMap = new HashMap<Integer, Long>();
    private RandomAccessFile dataRandomAccessFile;
    private DataOutputPrimitivesWriter dataPrimitivesWriter;
    private File dataFile;

    public DirectoryDataCacheWriter(File cacheDirectory) {
        File f = new File(cacheDirectory, "data.primitives");
        if (f.exists()) {
            f.delete();
        }
        try {
            this.dataRandomAccessFile = new RandomAccessFile(f, "rw");
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        this.dataPrimitivesWriter = new DataOutputPrimitivesWriter(dataRandomAccessFile);
        this.dataFile = f;
    }

    public void cacheFicheData(FicheData ficheData) {
        try {
            long position = dataRandomAccessFile.getFilePointer();
            int codefiche = ficheData.getCodeFiche();
            positionMap.put(codefiche, position);
            FicheData.toPrimitives(ficheData, dataPrimitivesWriter);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    public void cacheMotcleData(MotcleData motcleData) {
        try {
            long position = dataRandomAccessFile.getFilePointer();
            int codemotcle = motcleData.getCodeMotcle();
            positionMap.put(codemotcle, position);
            MotcleData.toPrimitives(motcleData, dataPrimitivesWriter);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    public DataCacheReaderFactory endCacheWrite() {
        try {
            dataPrimitivesWriter = null;
            dataRandomAccessFile.close();
            dataRandomAccessFile = null;
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        return new InternalDataCacheReaderFactory(positionMap, dataFile);
    }


    private static class InternalDataCacheReaderFactory implements DataCacheReaderFactory {

        private HashMap<Integer, Long> positionMap;
        private File dataFile;

        private InternalDataCacheReaderFactory(HashMap<Integer, Long> positionMap, File dataFile) {
            this.positionMap = positionMap;
            this.dataFile = dataFile;
        }

        public DataCacheReader newReader() {
            return new InternalDataCacheReader();
        }


        private class InternalDataCacheReader implements DataCacheReader {

            private RandomAccessFile dataRandomAccessFile;
            private DataInputPrimitivesReader dataPrimitivesReader;

            private InternalDataCacheReader() {
                try {
                    this.dataRandomAccessFile = new RandomAccessFile(dataFile, "r");
                } catch (IOException ioe) {
                    throw new SctStorageException(ioe);
                }
                this.dataPrimitivesReader = new DataInputPrimitivesReader(dataRandomAccessFile);
            }

            public FicheData getFicheData(int codefiche) {
                Long position = positionMap.get(codefiche);
                if (position == null) {
                    return null;
                }
                try {
                    dataRandomAccessFile.seek(position.longValue());
                    return FicheData.fromPrimitives(dataPrimitivesReader);
                } catch (IOException ioe) {
                    throw new SctStorageException(ioe);
                }
            }

            public MotcleData getMotcleData(int codemotcle) {
                Long position = positionMap.get(codemotcle);
                if (position == null) {
                    return null;
                }
                try {
                    dataRandomAccessFile.seek(position.longValue());
                    return MotcleData.fromPrimitives(dataPrimitivesReader);
                } catch (IOException ioe) {
                    throw new SctStorageException(ioe);
                }
            }

            public void close() {
                try {
                    dataPrimitivesReader = null;
                    dataRandomAccessFile.close();
                    dataRandomAccessFile = null;
                } catch (IOException ioe) {
                    throw new SctStorageException(ioe);
                }
            }

        }

    }

}
